"""
Python module generated from Java source file org.joml.FrustumIntersection

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class FrustumIntersection:
    """
    Efficiently performs frustum intersection tests by caching the frustum planes of an arbitrary transformation Matrix4fc matrix.
    
    This class is preferred over the frustum intersection methods in Matrix4fc when many objects need to be culled by the same static frustum.

    Author(s)
    - Kai Burjack
    """

    PLANE_NX = 0
    """
    Return value of .intersectAab(float, float, float, float, float, float) intersectAab()
    and its different overloads identifying the plane with equation `x=-1` when using the identity frustum.
    """
    PLANE_PX = 1
    """
    Return value of .intersectAab(float, float, float, float, float, float) intersectAab()
    and its different overloads identifying the plane with equation `x=1` when using the identity frustum.
    """
    PLANE_NY = 2
    """
    Return value of .intersectAab(float, float, float, float, float, float) intersectAab()
    and its different overloads identifying the plane with equation `y=-1` when using the identity frustum.
    """
    PLANE_PY = 3
    """
    Return value of .intersectAab(float, float, float, float, float, float) intersectAab()
    and its different overloads identifying the plane with equation `y=1` when using the identity frustum.
    """
    PLANE_NZ = 4
    """
    Return value of .intersectAab(float, float, float, float, float, float) intersectAab()
    and its different overloads identifying the plane with equation `z=-1` when using the identity frustum.
    """
    PLANE_PZ = 5
    """
    Return value of .intersectAab(float, float, float, float, float, float) intersectAab()
    and its different overloads identifying the plane with equation `z=1` when using the identity frustum.
    """
    INTERSECT = -1
    """
    Return value of .intersectAab(float, float, float, float, float, float) intersectAab()
    and its different overloads indicating that the axis-aligned box intersects the frustum.
    """
    INSIDE = -2
    """
    Return value of .intersectAab(float, float, float, float, float, float) intersectAab()
    and its different overloads indicating that the axis-aligned box is fully inside of the frustum.
    """
    OUTSIDE = -3
    """
    Return value of .intersectSphere(Vector3fc, float) or .intersectSphere(float, float, float, float)
    indicating that the sphere is completely outside of the frustum.
    """
    PLANE_MASK_NX = 1 << PLANE_NX
    """
    The value in a bitmask for
    .intersectAab(float, float, float, float, float, float, int) intersectAab()
    that identifies the plane with equation `x=-1` when using the identity frustum.
    """
    PLANE_MASK_PX = 1 << PLANE_PX
    """
    The value in a bitmask for
    .intersectAab(float, float, float, float, float, float, int) intersectAab()
    that identifies the plane with equation `x=1` when using the identity frustum.
    """
    PLANE_MASK_NY = 1 << PLANE_NY
    """
    The value in a bitmask for
    .intersectAab(float, float, float, float, float, float, int) intersectAab()
    that identifies the plane with equation `y=-1` when using the identity frustum.
    """
    PLANE_MASK_PY = 1 << PLANE_PY
    """
    The value in a bitmask for
    .intersectAab(float, float, float, float, float, float, int) intersectAab()
    that identifies the plane with equation `y=1` when using the identity frustum.
    """
    PLANE_MASK_NZ = 1 << PLANE_NZ
    """
    The value in a bitmask for
    .intersectAab(float, float, float, float, float, float, int) intersectAab()
    that identifies the plane with equation `z=-1` when using the identity frustum.
    """
    PLANE_MASK_PZ = 1 << PLANE_PZ
    """
    The value in a bitmask for
    .intersectAab(float, float, float, float, float, float, int) intersectAab()
    that identifies the plane with equation `z=1` when using the identity frustum.
    """


    def __init__(self):
        """
        Create a new FrustumIntersection with undefined frustum planes.
        
        Before using any of the frustum culling methods, make sure to define the frustum planes using .set(Matrix4fc).
        """
        ...


    def __init__(self, m: "Matrix4fc"):
        """
        Create a new FrustumIntersection from the given Matrix4fc matrix by extracing the matrix's frustum planes.
        
        In order to update the compute frustum planes later on, call .set(Matrix4fc).

        Arguments
        - m: the Matrix4fc to create the frustum culler from

        See
        - .set(Matrix4fc)
        """
        ...


    def __init__(self, m: "Matrix4fc", allowTestSpheres: bool):
        """
        Create a new FrustumIntersection from the given Matrix4fc matrix by extracing the matrix's frustum planes.
        
        In order to update the compute frustum planes later on, call .set(Matrix4fc).

        Arguments
        - m: the Matrix4fc to create the frustum culler from
        - allowTestSpheres: whether the methods .testSphere(Vector3fc, float), .testSphere(float, float, float, float),
                 .intersectSphere(Vector3fc, float) or .intersectSphere(float, float, float, float) will used.
                 If no spheres need to be tested, then `False` should be used

        See
        - .set(Matrix4fc)
        """
        ...


    def set(self, m: "Matrix4fc") -> "FrustumIntersection":
        """
        Update the stored frustum planes of `this` FrustumIntersection with the given Matrix4fc matrix.
        
        Reference: <a href="http://gamedevs.org/uploads/fast-extraction-viewing-frustum-planes-from-world-view-projection-matrix.pdf">
        Fast Extraction of Viewing Frustum Planes from the World-View-Projection Matrix</a>

        Arguments
        - m: the Matrix4fc matrix to update `this` frustum culler's frustum planes from

        Returns
        - this
        """
        ...


    def set(self, m: "Matrix4fc", allowTestSpheres: bool) -> "FrustumIntersection":
        """
        Update the stored frustum planes of `this` FrustumIntersection with the given Matrix4fc matrix and
        allow to optimize the frustum plane extraction in the case when no intersection test is needed for spheres.
        
        Reference: <a href="http://gamedevs.org/uploads/fast-extraction-viewing-frustum-planes-from-world-view-projection-matrix.pdf">
        Fast Extraction of Viewing Frustum Planes from the World-View-Projection Matrix</a>

        Arguments
        - m: the Matrix4fc matrix to update `this` frustum culler's frustum planes from
        - allowTestSpheres: whether the methods .testSphere(Vector3fc, float), .testSphere(float, float, float, float),
                 .intersectSphere(Vector3fc, float) or .intersectSphere(float, float, float, float) will be used.
                 If no spheres need to be tested, then `False` should be used

        Returns
        - this
        """
        ...


    def testPoint(self, point: "Vector3fc") -> bool:
        """
        Test whether the given point is within the frustum defined by `this` frustum culler.

        Arguments
        - point: the point to test

        Returns
        - `True` if the given point is inside the frustum; `False` otherwise
        """
        ...


    def testPoint(self, x: float, y: float, z: float) -> bool:
        """
        Test whether the given point `(x, y, z)` is within the frustum defined by `this` frustum culler.

        Arguments
        - x: the x-coordinate of the point
        - y: the y-coordinate of the point
        - z: the z-coordinate of the point

        Returns
        - `True` if the given point is inside the frustum; `False` otherwise
        """
        ...


    def testSphere(self, center: "Vector3fc", radius: float) -> bool:
        """
        Test whether the given sphere is partly or completely within or outside of the frustum defined by `this` frustum culler.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns `True` for spheres that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.

        Arguments
        - center: the sphere's center
        - radius: the sphere's radius

        Returns
        - `True` if the given sphere is partly or completely inside the frustum;
                `False` otherwise
        """
        ...


    def testSphere(self, x: float, y: float, z: float, r: float) -> bool:
        """
        Test whether the given sphere is partly or completely within or outside of the frustum defined by `this` frustum culler.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns `True` for spheres that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.

        Arguments
        - x: the x-coordinate of the sphere's center
        - y: the y-coordinate of the sphere's center
        - z: the z-coordinate of the sphere's center
        - r: the sphere's radius

        Returns
        - `True` if the given sphere is partly or completely inside the frustum;
                `False` otherwise
        """
        ...


    def intersectSphere(self, center: "Vector3fc", radius: float) -> int:
        """
        Determine whether the given sphere is partly or completely within or outside of the frustum defined by `this` frustum culler.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns .INTERSECT for spheres that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.

        Arguments
        - center: the sphere's center
        - radius: the sphere's radius

        Returns
        - .INSIDE if the given sphere is completely inside the frustum, or .INTERSECT if the sphere intersects
                the frustum, or .OUTSIDE if the sphere is outside of the frustum
        """
        ...


    def intersectSphere(self, x: float, y: float, z: float, r: float) -> int:
        """
        Determine whether the given sphere is partly or completely within or outside of the frustum defined by `this` frustum culler.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns .INTERSECT for spheres that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.

        Arguments
        - x: the x-coordinate of the sphere's center
        - y: the y-coordinate of the sphere's center
        - z: the z-coordinate of the sphere's center
        - r: the sphere's radius

        Returns
        - .INSIDE if the given sphere is completely inside the frustum, or .INTERSECT if the sphere intersects
                the frustum, or .OUTSIDE if the sphere is outside of the frustum
        """
        ...


    def testAab(self, min: "Vector3fc", max: "Vector3fc") -> bool:
        """
        Test whether the given axis-aligned box is partly or completely within or outside of the frustum defined by `this` frustum culler.
        The box is specified via its `min` and `max` corner coordinates.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns `True` for boxes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.

        Arguments
        - min: the minimum corner coordinates of the axis-aligned box
        - max: the maximum corner coordinates of the axis-aligned box

        Returns
        - `True` if the axis-aligned box is completely or partly inside of the frustum; `False` otherwise
        """
        ...


    def testAab(self, minX: float, minY: float, minZ: float, maxX: float, maxY: float, maxZ: float) -> bool:
        """
        Test whether the given axis-aligned box is partly or completely within or outside of the frustum defined by `this` frustum culler.
        The box is specified via its min and max corner coordinates.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns `True` for boxes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.
        
        Reference: <a href="http://old.cescg.org/CESCG-2002/DSykoraJJelinek/">Efficient View Frustum Culling</a>

        Arguments
        - minX: the x-coordinate of the minimum corner
        - minY: the y-coordinate of the minimum corner
        - minZ: the z-coordinate of the minimum corner
        - maxX: the x-coordinate of the maximum corner
        - maxY: the y-coordinate of the maximum corner
        - maxZ: the z-coordinate of the maximum corner

        Returns
        - `True` if the axis-aligned box is completely or partly inside of the frustum; `False` otherwise
        """
        ...


    def testPlaneXY(self, min: "Vector2fc", max: "Vector2fc") -> bool:
        """
        Test whether the given XY-plane (at `Z = 0`) is partly or completely within or outside of the frustum defined by `this` frustum culler.
        The plane is specified via its `min` and `max` corner coordinates.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns `True` for planes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.

        Arguments
        - min: the minimum corner coordinates of the XY-plane
        - max: the maximum corner coordinates of the XY-plane

        Returns
        - `True` if the XY-plane is completely or partly inside of the frustum; `False` otherwise
        """
        ...


    def testPlaneXY(self, minX: float, minY: float, maxX: float, maxY: float) -> bool:
        """
        Test whether the given XY-plane (at `Z = 0`) is partly or completely within or outside of the frustum defined by `this` frustum culler.
        The plane is specified via its min and max corner coordinates.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns `True` for planes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.
        
        Reference: <a href="http://old.cescg.org/CESCG-2002/DSykoraJJelinek/">Efficient View Frustum Culling</a>

        Arguments
        - minX: the x-coordinate of the minimum corner
        - minY: the y-coordinate of the minimum corner
        - maxX: the x-coordinate of the maximum corner
        - maxY: the y-coordinate of the maximum corner

        Returns
        - `True` if the XY-plane is completely or partly inside of the frustum; `False` otherwise
        """
        ...


    def testPlaneXZ(self, minX: float, minZ: float, maxX: float, maxZ: float) -> bool:
        """
        Test whether the given XZ-plane (at `Y = 0`) is partly or completely within or outside of the frustum defined by `this` frustum culler.
        The plane is specified via its min and max corner coordinates.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns `True` for planes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.
        
        Reference: <a href="http://old.cescg.org/CESCG-2002/DSykoraJJelinek/">Efficient View Frustum Culling</a>

        Arguments
        - minX: the x-coordinate of the minimum corner
        - minZ: the z-coordinate of the minimum corner
        - maxX: the x-coordinate of the maximum corner
        - maxZ: the z-coordinate of the maximum corner

        Returns
        - `True` if the XZ-plane is completely or partly inside of the frustum; `False` otherwise
        """
        ...


    def intersectAab(self, min: "Vector3fc", max: "Vector3fc") -> int:
        """
        Determine whether the given axis-aligned box is partly or completely within or outside of the frustum defined by `this` frustum culler
        and, if the box is not inside this frustum, return the index of the plane that culled it.
        The box is specified via its `min` and `max` corner coordinates.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns .INTERSECT for boxes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.

        Arguments
        - min: the minimum corner coordinates of the axis-aligned box
        - max: the maximum corner coordinates of the axis-aligned box

        Returns
        - the index of the first plane that culled the box, if the box does not intersect the frustum;
                or .INTERSECT if the box intersects the frustum, or .INSIDE if the box is fully inside of the frustum.
                The plane index is one of .PLANE_NX, .PLANE_PX, .PLANE_NY, .PLANE_PY, .PLANE_NZ and .PLANE_PZ
        """
        ...


    def intersectAab(self, minX: float, minY: float, minZ: float, maxX: float, maxY: float, maxZ: float) -> int:
        """
        Determine whether the given axis-aligned box is partly or completely within or outside of the frustum defined by `this` frustum culler
        and, if the box is not inside this frustum, return the index of the plane that culled it.
        The box is specified via its min and max corner coordinates.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns .INTERSECT for boxes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.
        
        Reference: <a href="http://old.cescg.org/CESCG-2002/DSykoraJJelinek/">Efficient View Frustum Culling</a>

        Arguments
        - minX: the x-coordinate of the minimum corner
        - minY: the y-coordinate of the minimum corner
        - minZ: the z-coordinate of the minimum corner
        - maxX: the x-coordinate of the maximum corner
        - maxY: the y-coordinate of the maximum corner
        - maxZ: the z-coordinate of the maximum corner

        Returns
        - the index of the first plane that culled the box, if the box does not intersect the frustum,
                or .INTERSECT if the box intersects the frustum, or .INSIDE if the box is fully inside of the frustum.
                The plane index is one of .PLANE_NX, .PLANE_PX, .PLANE_NY, .PLANE_PY, .PLANE_NZ and .PLANE_PZ
        """
        ...


    def distanceToPlane(self, minX: float, minY: float, minZ: float, maxX: float, maxY: float, maxZ: float, plane: int) -> float:
        """
        Compute the signed distance from the given axis-aligned box to the `plane`.

        Arguments
        - minX: the x-coordinate of the minimum corner
        - minY: the y-coordinate of the minimum corner
        - minZ: the z-coordinate of the minimum corner
        - maxX: the x-coordinate of the maximum corner
        - maxY: the y-coordinate of the maximum corner
        - maxZ: the z-coordinate of the maximum corner
        - plane: one of 
                 .PLANE_NX, .PLANE_PX,
                 .PLANE_NY, .PLANE_PY, 
                 .PLANE_NZ and .PLANE_PZ

        Returns
        - the signed distance of the axis-aligned box to the plane
        """
        ...


    def intersectAab(self, min: "Vector3fc", max: "Vector3fc", mask: int) -> int:
        """
        Determine whether the given axis-aligned box is partly or completely within or outside of the frustum defined by `this` frustum culler
        and, if the box is not inside this frustum, return the index of the plane that culled it.
        The box is specified via its `min` and `max` corner coordinates.
        
        This method differs from .intersectAab(Vector3fc, Vector3fc) in that
        it allows to mask-off planes that should not be calculated. For example, in order to only test a box against the
        left frustum plane, use a mask of .PLANE_MASK_NX. Or in order to test all planes *except* the left plane, use 
        a mask of `(~0 ^ PLANE_MASK_NX)`.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns .INTERSECT for boxes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.

        Arguments
        - min: the minimum corner coordinates of the axis-aligned box
        - max: the maximum corner coordinates of the axis-aligned box
        - mask: contains as bitset all the planes that should be tested.
                 This value can be any combination of 
                 .PLANE_MASK_NX, .PLANE_MASK_PX,
                 .PLANE_MASK_NY, .PLANE_MASK_PY, 
                 .PLANE_MASK_NZ and .PLANE_MASK_PZ

        Returns
        - the index of the first plane that culled the box, if the box does not intersect the frustum,
                or .INTERSECT if the box intersects the frustum, or .INSIDE if the box is fully inside of the frustum.
                The plane index is one of .PLANE_NX, .PLANE_PX, .PLANE_NY, .PLANE_PY, .PLANE_NZ and .PLANE_PZ
        """
        ...


    def intersectAab(self, minX: float, minY: float, minZ: float, maxX: float, maxY: float, maxZ: float, mask: int) -> int:
        """
        Determine whether the given axis-aligned box is partly or completely within or outside of the frustum defined by `this` frustum culler
        and, if the box is not inside this frustum, return the index of the plane that culled it.
        The box is specified via its min and max corner coordinates.
        
        This method differs from .intersectAab(float, float, float, float, float, float) in that
        it allows to mask-off planes that should not be calculated. For example, in order to only test a box against the
        left frustum plane, use a mask of .PLANE_MASK_NX. Or in order to test all planes *except* the left plane, use 
        a mask of `(~0 ^ PLANE_MASK_NX)`.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns .INTERSECT for boxes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.
        
        Reference: <a href="http://old.cescg.org/CESCG-2002/DSykoraJJelinek/">Efficient View Frustum Culling</a>

        Arguments
        - minX: the x-coordinate of the minimum corner
        - minY: the y-coordinate of the minimum corner
        - minZ: the z-coordinate of the minimum corner
        - maxX: the x-coordinate of the maximum corner
        - maxY: the y-coordinate of the maximum corner
        - maxZ: the z-coordinate of the maximum corner
        - mask: contains as bitset all the planes that should be tested.
                 This value can be any combination of 
                 .PLANE_MASK_NX, .PLANE_MASK_PX,
                 .PLANE_MASK_NY, .PLANE_MASK_PY, 
                 .PLANE_MASK_NZ and .PLANE_MASK_PZ

        Returns
        - the index of the first plane that culled the box, if the box does not intersect the frustum,
                or .INTERSECT if the box intersects the frustum, or .INSIDE if the box is fully inside of the frustum.
                The plane index is one of .PLANE_NX, .PLANE_PX, .PLANE_NY, .PLANE_PY, .PLANE_NZ and .PLANE_PZ
        """
        ...


    def intersectAab(self, min: "Vector3fc", max: "Vector3fc", mask: int, startPlane: int) -> int:
        """
        Determine whether the given axis-aligned box is partly or completely within or outside of the frustum defined by `this` frustum culler
        and, if the box is not inside this frustum, return the index of the plane that culled it.
        The box is specified via its `min` and `max` corner coordinates.
        
        This method differs from .intersectAab(Vector3fc, Vector3fc) in that
        it allows to mask-off planes that should not be calculated. For example, in order to only test a box against the
        left frustum plane, use a mask of .PLANE_MASK_NX. Or in order to test all planes *except* the left plane, use 
        a mask of `(~0 ^ PLANE_MASK_NX)`.
        
        In addition, the `startPlane` denotes the first frustum plane to test the box against. To use this effectively means to store the
        plane that previously culled an axis-aligned box (as returned by `intersectAab()`) and in the next frame use the return value
        as the argument to the `startPlane` parameter of this method. The assumption is that the plane that culled the object previously will also
        cull it now (temporal coherency) and the culling computation is likely reduced in that case.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns .INTERSECT for boxes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.

        Arguments
        - min: the minimum corner coordinates of the axis-aligned box
        - max: the maximum corner coordinates of the axis-aligned box
        - mask: contains as bitset all the planes that should be tested.
                 This value can be any combination of 
                 .PLANE_MASK_NX, .PLANE_MASK_PX,
                 .PLANE_MASK_NY, .PLANE_MASK_PY, 
                 .PLANE_MASK_NZ and .PLANE_MASK_PZ
        - startPlane: the first frustum plane to test the axis-aligned box against. It is one of
                 .PLANE_NX, .PLANE_PX, .PLANE_NY, .PLANE_PY, .PLANE_NZ and .PLANE_PZ

        Returns
        - the index of the first plane that culled the box, if the box does not intersect the frustum,
                or .INTERSECT if the box intersects the frustum, or .INSIDE if the box is fully inside of the frustum.
                The plane index is one of .PLANE_NX, .PLANE_PX, .PLANE_NY, .PLANE_PY, .PLANE_NZ and .PLANE_PZ
        """
        ...


    def intersectAab(self, minX: float, minY: float, minZ: float, maxX: float, maxY: float, maxZ: float, mask: int, startPlane: int) -> int:
        """
        Determine whether the given axis-aligned box is partly or completely within or outside of the frustum defined by `this` frustum culler
        and, if the box is not inside this frustum, return the index of the plane that culled it.
        The box is specified via its min and max corner coordinates.
        
        This method differs from .intersectAab(float, float, float, float, float, float) in that
        it allows to mask-off planes that should not be calculated. For example, in order to only test a box against the
        left frustum plane, use a mask of .PLANE_MASK_NX. Or in order to test all planes *except* the left plane, use 
        a mask of `(~0 ^ PLANE_MASK_NX)`.
        
        In addition, the `startPlane` denotes the first frustum plane to test the box against. To use this effectively means to store the
        plane that previously culled an axis-aligned box (as returned by `intersectAab()`) and in the next frame use the return value
        as the argument to the `startPlane` parameter of this method. The assumption is that the plane that culled the object previously will also
        cull it now (temporal coherency) and the culling computation is likely reduced in that case.
        
        The algorithm implemented by this method is conservative. This means that in certain circumstances a *False positive*
        can occur, when the method returns .INTERSECT for boxes that do not intersect the frustum.
        See <a href="http://iquilezles.org/www/articles/frustumcorrect/frustumcorrect.htm">iquilezles.org</a> for an examination of this problem.
        
        Reference: <a href="http://old.cescg.org/CESCG-2002/DSykoraJJelinek/">Efficient View Frustum Culling</a>

        Arguments
        - minX: the x-coordinate of the minimum corner
        - minY: the y-coordinate of the minimum corner
        - minZ: the z-coordinate of the minimum corner
        - maxX: the x-coordinate of the maximum corner
        - maxY: the y-coordinate of the maximum corner
        - maxZ: the z-coordinate of the maximum corner
        - mask: contains as bitset all the planes that should be tested.
                 This value can be any combination of 
                 .PLANE_MASK_NX, .PLANE_MASK_PX,
                 .PLANE_MASK_NY, .PLANE_MASK_PY, 
                 .PLANE_MASK_NZ and .PLANE_MASK_PZ
        - startPlane: the first frustum plane to test the axis-aligned box against. It is one of
                 .PLANE_NX, .PLANE_PX, .PLANE_NY, .PLANE_PY, .PLANE_NZ and .PLANE_PZ

        Returns
        - the index of the first plane that culled the box, if the box does not intersect the frustum,
                or .INTERSECT if the box intersects the frustum, or .INSIDE if the box is fully inside of the frustum.
                The plane index is one of .PLANE_NX, .PLANE_PX, .PLANE_NY, .PLANE_PY, .PLANE_NZ and .PLANE_PZ
        """
        ...


    def testLineSegment(self, a: "Vector3fc", b: "Vector3fc") -> bool:
        """
        Test whether the given line segment, defined by the end points `a` and `b`, 
        is partly or completely within the frustum defined by `this` frustum culler.

        Arguments
        - a: the line segment's first end point
        - b: the line segment's second end point

        Returns
        - `True` if the given line segment is partly or completely inside the frustum;
                `False` otherwise
        """
        ...


    def testLineSegment(self, aX: float, aY: float, aZ: float, bX: float, bY: float, bZ: float) -> bool:
        """
        Test whether the given line segment, defined by the end points `(aX, aY, aZ)` and `(bX, bY, bZ)`, 
        is partly or completely within the frustum defined by `this` frustum culler.

        Arguments
        - aX: the x coordinate of the line segment's first end point
        - aY: the y coordinate of the line segment's first end point
        - aZ: the z coordinate of the line segment's first end point
        - bX: the x coordinate of the line segment's second end point
        - bY: the y coordinate of the line segment's second end point
        - bZ: the z coordinate of the line segment's second end point

        Returns
        - `True` if the given line segment is partly or completely inside the frustum;
                `False` otherwise
        """
        ...



"""
Python module generated from Java source file com.google.gson.FieldNamingPolicy

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import *
from enum import Enum
from java.lang.reflect import Field
from java.util import Locale
from typing import Any, Callable, Iterable, Tuple


class FieldNamingPolicy(Enum):
    """
    An enumeration that defines a few standard naming conventions for JSON field names. This
    enumeration should be used in conjunction with com.google.gson.GsonBuilder to configure a
    com.google.gson.Gson instance to properly translate Java field names into the desired
    JSON field names.

    Author(s)
    - Joel Leitch
    """

    IDENTITY = 0
    """
    Using this naming policy with Gson will ensure that the field name is unchanged.
    """
    UPPER_CAMEL_CASE = 1
    """
    Using this naming policy with Gson will ensure that the first "letter" of the Java field name
    is capitalized when serialized to its JSON form.
    
    Here are a few examples of the form "Java Field Name" ---&gt; "JSON Field Name":
    
    
      - someFieldName ---&gt; SomeFieldName
      - _someFieldName ---&gt; _SomeFieldName
    """
    UPPER_CAMEL_CASE_WITH_SPACES = 2
    """
    Using this naming policy with Gson will ensure that the first "letter" of the Java field name
    is capitalized when serialized to its JSON form and the words will be separated by a space.
    
    Here are a few examples of the form "Java Field Name" ---&gt; "JSON Field Name":
    
    
      - someFieldName ---&gt; Some Field Name
      - _someFieldName ---&gt; _Some Field Name

    Since
    - 1.4
    """
    UPPER_CASE_WITH_UNDERSCORES = 3
    """
    Using this naming policy with Gson will modify the Java Field name from its camel cased form to
    an upper case field name where each word is separated by an underscore (_).
    
    Here are a few examples of the form "Java Field Name" ---&gt; "JSON Field Name":
    
    
      - someFieldName ---&gt; SOME_FIELD_NAME
      - _someFieldName ---&gt; _SOME_FIELD_NAME
      - aStringField ---&gt; A_STRING_FIELD
      - aURL ---&gt; A_U_R_L

    Since
    - 2.9.0
    """
    LOWER_CASE_WITH_UNDERSCORES = 4
    """
    Using this naming policy with Gson will modify the Java Field name from its camel cased form to
    a lower case field name where each word is separated by an underscore (_).
    
    Here are a few examples of the form "Java Field Name" ---&gt; "JSON Field Name":
    
    
      - someFieldName ---&gt; some_field_name
      - _someFieldName ---&gt; _some_field_name
      - aStringField ---&gt; a_string_field
      - aURL ---&gt; a_u_r_l
    """
    LOWER_CASE_WITH_DASHES = 5
    """
    Using this naming policy with Gson will modify the Java Field name from its camel cased form to
    a lower case field name where each word is separated by a dash (-).
    
    Here are a few examples of the form "Java Field Name" ---&gt; "JSON Field Name":
    
    
      - someFieldName ---&gt; some-field-name
      - _someFieldName ---&gt; _some-field-name
      - aStringField ---&gt; a-string-field
      - aURL ---&gt; a-u-r-l
    
    
    Using dashes in JavaScript is not recommended since dash is also used for a minus sign in
    expressions. This requires that a field named with dashes is always accessed as a quoted
    property like `myobject['my-field']`. Accessing it as an object field `myobject.my-field` will result in an unintended JavaScript expression.

    Since
    - 1.4
    """
    LOWER_CASE_WITH_DOTS = 6
    """
    Using this naming policy with Gson will modify the Java Field name from its camel cased form to
    a lower case field name where each word is separated by a dot (.).
    
    Here are a few examples of the form "Java Field Name" ---&gt; "JSON Field Name":
    
    
      - someFieldName ---&gt; some.field.name
      - _someFieldName ---&gt; _some.field.name
      - aStringField ---&gt; a.string.field
      - aURL ---&gt; a.u.r.l
    
    
    Using dots in JavaScript is not recommended since dot is also used for a member sign in
    expressions. This requires that a field named with dots is always accessed as a quoted property
    like `myobject['my.field']`. Accessing it as an object field `myobject.my.field`
    will result in an unintended JavaScript expression.

    Since
    - 2.8.4
    """



"""
Python module generated from Java source file java.util.ListIterator

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from typing import Any, Callable, Iterable, Tuple


class ListIterator(Iterator):
    """
    An iterator for lists that allows the programmer
    to traverse the list in either direction, modify
    the list during iteration, and obtain the iterator's
    current position in the list. A `ListIterator`
    has no current element; its <I>cursor position</I> always
    lies between the element that would be returned by a call
    to `previous()` and the element that would be
    returned by a call to `next()`.
    An iterator for a list of length `n` has `n+1` possible
    cursor positions, as illustrated by the carets (`^`) below:
    <PRE>
                         Element(0)   Element(1)   Element(2)   ... Element(n-1)
    cursor positions:  ^            ^            ^            ^                  ^
    </PRE>
    Note that the .remove and .set(Object) methods are
    *not* defined in terms of the cursor position;  they are defined to
    operate on the last element returned by a call to .next or
    .previous().
    
    This interface is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.

    Author(s)
    - Josh Bloch

    See
    - List.listIterator()

    Since
    - 1.2
    """

    def hasNext(self) -> bool:
        """
        Returns `True` if this list iterator has more elements when
        traversing the list in the forward direction. (In other words,
        returns `True` if .next would return an element rather
        than throwing an exception.)

        Returns
        - `True` if the list iterator has more elements when
                traversing the list in the forward direction
        """
        ...


    def next(self) -> "E":
        """
        Returns the next element in the list and advances the cursor position.
        This method may be called repeatedly to iterate through the list,
        or intermixed with calls to .previous to go back and forth.
        (Note that alternating calls to `next` and `previous`
        will return the same element repeatedly.)

        Returns
        - the next element in the list

        Raises
        - NoSuchElementException: if the iteration has no next element
        """
        ...


    def hasPrevious(self) -> bool:
        """
        Returns `True` if this list iterator has more elements when
        traversing the list in the reverse direction.  (In other words,
        returns `True` if .previous would return an element
        rather than throwing an exception.)

        Returns
        - `True` if the list iterator has more elements when
                traversing the list in the reverse direction
        """
        ...


    def previous(self) -> "E":
        """
        Returns the previous element in the list and moves the cursor
        position backwards.  This method may be called repeatedly to
        iterate through the list backwards, or intermixed with calls to
        .next to go back and forth.  (Note that alternating calls
        to `next` and `previous` will return the same
        element repeatedly.)

        Returns
        - the previous element in the list

        Raises
        - NoSuchElementException: if the iteration has no previous
                element
        """
        ...


    def nextIndex(self) -> int:
        """
        Returns the index of the element that would be returned by a
        subsequent call to .next. (Returns list size if the list
        iterator is at the end of the list.)

        Returns
        - the index of the element that would be returned by a
                subsequent call to `next`, or list size if the list
                iterator is at the end of the list
        """
        ...


    def previousIndex(self) -> int:
        """
        Returns the index of the element that would be returned by a
        subsequent call to .previous. (Returns -1 if the list
        iterator is at the beginning of the list.)

        Returns
        - the index of the element that would be returned by a
                subsequent call to `previous`, or -1 if the list
                iterator is at the beginning of the list
        """
        ...


    def remove(self) -> None:
        """
        Removes from the list the last element that was returned by .next or .previous (optional operation).  This call can
        only be made once per call to `next` or `previous`.
        It can be made only if .add has not been
        called after the last call to `next` or `previous`.

        Raises
        - UnsupportedOperationException: if the `remove`
                operation is not supported by this list iterator
        - IllegalStateException: if neither `next` nor
                `previous` have been called, or `remove` or
                `add` have been called after the last call to
                `next` or `previous`
        """
        ...


    def set(self, e: "E") -> None:
        """
        Replaces the last element returned by .next or
        .previous with the specified element (optional operation).
        This call can be made only if neither .remove nor .add have been called after the last call to `next` or
        `previous`.

        Arguments
        - e: the element with which to replace the last element returned by
                 `next` or `previous`

        Raises
        - UnsupportedOperationException: if the `set` operation
                is not supported by this list iterator
        - ClassCastException: if the class of the specified element
                prevents it from being added to this list
        - IllegalArgumentException: if some aspect of the specified
                element prevents it from being added to this list
        - IllegalStateException: if neither `next` nor
                `previous` have been called, or `remove` or
                `add` have been called after the last call to
                `next` or `previous`
        """
        ...


    def add(self, e: "E") -> None:
        """
        Inserts the specified element into the list (optional operation).
        The element is inserted immediately before the element that
        would be returned by .next, if any, and after the element
        that would be returned by .previous, if any.  (If the
        list contains no elements, the new element becomes the sole element
        on the list.)  The new element is inserted before the implicit
        cursor: a subsequent call to `next` would be unaffected, and a
        subsequent call to `previous` would return the new element.
        (This call increases by one the value that would be returned by a
        call to `nextIndex` or `previousIndex`.)

        Arguments
        - e: the element to insert

        Raises
        - UnsupportedOperationException: if the `add` method is
                not supported by this list iterator
        - ClassCastException: if the class of the specified element
                prevents it from being added to this list
        - IllegalArgumentException: if some aspect of this element
                prevents it from being added to this list
        """
        ...



"""
Python module generated from Java source file org.bukkit.boss.BarStyle

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.boss import *
from typing import Any, Callable, Iterable, Tuple


class BarStyle(Enum):

    SOLID = 0
    """
    Makes the boss bar solid (no segments)
    """
    SEGMENTED_6 = 1
    """
    Splits the boss bar into 6 segments
    """
    SEGMENTED_10 = 2
    """
    Splits the boss bar into 10 segments
    """
    SEGMENTED_12 = 3
    """
    Splits the boss bar into 12 segments
    """
    SEGMENTED_20 = 4
    """
    Splits the boss bar into 20 segments
    """



"""
Python module generated from Java source file org.yaml.snakeyaml.error.YAMLException

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.error import *
from typing import Any, Callable, Iterable, Tuple


class YAMLException(RuntimeException):

    def __init__(self, message: str):
        ...


    def __init__(self, cause: "Throwable"):
        ...


    def __init__(self, message: str, cause: "Throwable"):
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.EnderDragonChangePhaseEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.entity import EnderDragon
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EnderDragonChangePhaseEvent(EntityEvent, Cancellable):
    """
    Called when an EnderDragon switches controller phase.
    """

    def __init__(self, enderDragon: "EnderDragon", currentPhase: "EnderDragon.Phase", newPhase: "EnderDragon.Phase"):
        ...


    def getEntity(self) -> "EnderDragon":
        ...


    def getCurrentPhase(self) -> "EnderDragon.Phase":
        """
        Gets the current phase that the dragon is in. This method will return null
        when a dragon is first spawned and hasn't yet been assigned a phase.

        Returns
        - the current dragon phase
        """
        ...


    def getNewPhase(self) -> "EnderDragon.Phase":
        """
        Gets the new phase that the dragon will switch to.

        Returns
        - the new dragon phase
        """
        ...


    def setNewPhase(self, newPhase: "EnderDragon.Phase") -> None:
        """
        Sets the new phase for the ender dragon.

        Arguments
        - newPhase: the new dragon phase
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.reflect.TypeCapture

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.reflect import *
from java.lang.reflect import ParameterizedType
from java.lang.reflect import Type
from typing import Any, Callable, Iterable, Tuple


class TypeCapture:
    """
    Captures the actual type of `T`.

    Author(s)
    - Ben Yu
    """





"""
Python module generated from Java source file com.google.common.util.concurrent.ListenableFuture

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import DoNotMock
from java.util.concurrent import Executor
from java.util.concurrent import Future
from java.util.concurrent import RejectedExecutionException
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ListenableFuture(Future):

    def addListener(self, listener: "Runnable", executor: "Executor") -> None:
        """
        Registers a listener to be Executor.execute(Runnable) run on the given executor.
        The listener will run when the `Future`'s computation is Future.isDone()
        complete or, if the computation is already complete, immediately.
        
        There is no guaranteed ordering of execution of listeners, but any listener added through
        this method is guaranteed to be called once the computation is complete.
        
        Exceptions thrown by a listener will be propagated up to the executor. Any exception thrown
        during `Executor.execute` (e.g., a `RejectedExecutionException` or an exception
        thrown by MoreExecutors.directExecutor direct execution) will be caught and
        logged.
        
        Note: If your listener is lightweight -- and will not cause stack overflow by completing
        more futures or adding more `directExecutor()` listeners inline -- consider MoreExecutors.directExecutor. Otherwise, avoid it: See the warnings on the docs for `directExecutor`.
        
        This is the most general listener interface. For common operations performed using
        listeners, see Futures. For a simplified but general listener interface, see Futures.addCallback addCallback().
        
        Memory consistency effects: Actions in a thread prior to adding a listener <a
        href="https://docs.oracle.com/javase/specs/jls/se7/html/jls-17.html#jls-17.4.5">
        *happen-before*</a> its execution begins, perhaps in another thread.
        
        Guava implementations of `ListenableFuture` promptly release references to listeners
        after executing them.

        Arguments
        - listener: the listener to run when the computation is complete
        - executor: the executor to run the listener in

        Raises
        - RejectedExecutionException: if we tried to execute the listener immediately but the
            executor rejected it.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerResourcePackStatusEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from java.util import UUID
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerResourcePackStatusEvent(PlayerEvent):
    """
    Called when a player takes action on a resource pack request sent via
    Player.setResourcePack(java.lang.String).
    """

    def __init__(self, who: "Player", id: "UUID", resourcePackStatus: "Status"):
        ...


    def getID(self) -> "UUID":
        """
        Gets the unique ID of this pack.

        Returns
        - unique resource pack ID.
        """
        ...


    def getStatus(self) -> "Status":
        """
        Gets the status of this pack.

        Returns
        - the current status
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class Status(Enum):
        """
        Status of the resource pack.
        """

        SUCCESSFULLY_LOADED = 0
        """
        The resource pack has been successfully downloaded and applied to the
        client.
        """
        DECLINED = 1
        """
        The client refused to accept the resource pack.
        """
        FAILED_DOWNLOAD = 2
        """
        The client accepted the pack, but download failed.
        """
        ACCEPTED = 3
        """
        The client accepted the pack and is beginning a download of it.
        """
        DOWNLOADED = 4
        """
        The client successfully downloaded the pack.
        """
        INVALID_URL = 5
        """
        The pack URL was invalid.
        """
        FAILED_RELOAD = 6
        """
        The client was unable to reload the pack.
        """
        DISCARDED = 7
        """
        The pack was discarded by the client.
        """



"""
Python module generated from Java source file com.google.common.collect.ImmutableEnumSet

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from com.google.errorprone.annotations.concurrent import LazyInit
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.io import Serializable
from java.util import EnumSet
from java.util import Spliterator
from java.util.function import Consumer
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class ImmutableEnumSet(ImmutableSet):
    """
    Implementation of ImmutableSet backed by a non-empty java.util.EnumSet.

    Author(s)
    - Jared Levy
    """

    def iterator(self) -> "UnmodifiableIterator"["E"]:
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        ...


    def forEach(self, action: "Consumer"["E"]) -> None:
        ...


    def size(self) -> int:
        ...


    def contains(self, object: "Object") -> bool:
        ...


    def containsAll(self, collection: Iterable[Any]) -> bool:
        ...


    def isEmpty(self) -> bool:
        ...


    def equals(self, object: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file com.google.common.collect.CollectCollectors

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import Preconditions
from com.google.common.collect import *
from java.util import Comparator
from java.util import EnumMap
from java.util import EnumSet
from java.util.function import BinaryOperator
from java.util.function import Function
from java.util.function import Supplier
from java.util.function import ToIntFunction
from java.util.stream import Collector
from java.util.stream import Collectors
from java.util.stream import Stream
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class CollectCollectors:
    """
    Collectors utilities for `common.collect` internals.
    """





"""
Python module generated from Java source file org.yaml.snakeyaml.DumperOptions

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from java.util import TimeZone
from org.yaml.snakeyaml import *
from org.yaml.snakeyaml.emitter import Emitter
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.serializer import AnchorGenerator
from org.yaml.snakeyaml.serializer import NumberAnchorGenerator
from typing import Any, Callable, Iterable, Tuple


class DumperOptions:
    """
    Configuration for serialisation
    """

    def isAllowUnicode(self) -> bool:
        """
        getter

        Returns
        - False when non-ASCII is escaped
        """
        ...


    def setAllowUnicode(self, allowUnicode: bool) -> None:
        """
        Specify whether to emit non-ASCII printable Unicode characters. The default value is True. When
        set to False then printable non-ASCII characters (Cyrillic, Chinese etc) will be not printed
        but escaped (to support ASCII terminals)

        Arguments
        - allowUnicode: if allowUnicode is False then all non-ASCII characters are escaped
        """
        ...


    def getDefaultScalarStyle(self) -> "ScalarStyle":
        """
        getter

        Returns
        - scalar style
        """
        ...


    def setDefaultScalarStyle(self, defaultStyle: "ScalarStyle") -> None:
        """
        Set default style for scalars. See YAML 1.1 specification, 2.3 Scalars
        (http://yaml.org/spec/1.1/#id858081)

        Arguments
        - defaultStyle: set the style for all scalars
        """
        ...


    def setIndent(self, indent: int) -> None:
        """
        Define indentation. Must be within the limits (1-10)

        Arguments
        - indent: number of spaces to serve as indentation
        """
        ...


    def getIndent(self) -> int:
        """
        getter

        Returns
        - indent
        """
        ...


    def setIndicatorIndent(self, indicatorIndent: int) -> None:
        """
        Set number of white spaces to use for the sequence indicator '-'

        Arguments
        - indicatorIndent: value to be used as indent
        """
        ...


    def getIndicatorIndent(self) -> int:
        ...


    def getIndentWithIndicator(self) -> bool:
        ...


    def setIndentWithIndicator(self, indentWithIndicator: bool) -> None:
        """
        Set to True to add the indent for sequences to the general indent

        Arguments
        - indentWithIndicator: - True when indent for sequences is added to general
        """
        ...


    def setVersion(self, version: "Version") -> None:
        """
        Of no use - it is better not to include YAML version as the directive

        Arguments
        - version: 1.0 or 1.1
        """
        ...


    def getVersion(self) -> "Version":
        """
        getter

        Returns
        - the expected version
        """
        ...


    def setCanonical(self, canonical: bool) -> None:
        """
        Force the emitter to produce a canonical YAML document.

        Arguments
        - canonical: True produce canonical YAML document
        """
        ...


    def isCanonical(self) -> bool:
        """
        getter

        Returns
        - True when well established format should be dumped
        """
        ...


    def setPrettyFlow(self, prettyFlow: bool) -> None:
        """
        Force the emitter to produce a pretty YAML document when using the flow style.

        Arguments
        - prettyFlow: True produce pretty flow YAML document
        """
        ...


    def isPrettyFlow(self) -> bool:
        """
        getter

        Returns
        - True for pretty style
        """
        ...


    def setWidth(self, bestWidth: int) -> None:
        """
        Specify the preferred width to emit scalars. When the scalar representation takes more then the
        preferred with the scalar will be split into a few lines. The default is 80.

        Arguments
        - bestWidth: the preferred width for scalars.
        """
        ...


    def getWidth(self) -> int:
        """
        getter

        Returns
        - the preferred width for scalars
        """
        ...


    def setSplitLines(self, splitLines: bool) -> None:
        """
        Specify whether to split lines exceeding preferred width for scalars. The default is True.

        Arguments
        - splitLines: whether to split lines exceeding preferred width for scalars.
        """
        ...


    def getSplitLines(self) -> bool:
        """
        getter

        Returns
        - True when to split lines exceeding preferred width for scalars
        """
        ...


    def getLineBreak(self) -> "LineBreak":
        """
        getter

        Returns
        - line break to separate lines
        """
        ...


    def setDefaultFlowStyle(self, defaultFlowStyle: "FlowStyle") -> None:
        """
        setter

        Arguments
        - defaultFlowStyle: - enum for the flow style
        """
        ...


    def getDefaultFlowStyle(self) -> "FlowStyle":
        """
        getter

        Returns
        - flow style for collections
        """
        ...


    def setLineBreak(self, lineBreak: "LineBreak") -> None:
        """
        Specify the line break to separate the lines. It is platform specific: Windows - "\r\n", old
        MacOS - "\r", Unix - "\n". The default value is the one for Unix.

        Arguments
        - lineBreak: to be used for the input
        """
        ...


    def isExplicitStart(self) -> bool:
        """
        getter

        Returns
        - True when '---' must be printed
        """
        ...


    def setExplicitStart(self, explicitStart: bool) -> None:
        """
        setter - require explicit '...'

        Arguments
        - explicitStart: - True to emit '---'
        """
        ...


    def isExplicitEnd(self) -> bool:
        """
        getter

        Returns
        - True when '...' must be printed
        """
        ...


    def setExplicitEnd(self, explicitEnd: bool) -> None:
        """
        setter - require explicit '...'

        Arguments
        - explicitEnd: - True to emit '...'
        """
        ...


    def getTags(self) -> dict[str, str]:
        """
        getter

        Returns
        - previously defined tag directives
        """
        ...


    def setTags(self, tags: dict[str, str]) -> None:
        """
        setter

        Arguments
        - tags: - tag directives for the YAML document
        """
        ...


    def isAllowReadOnlyProperties(self) -> bool:
        """
        Report whether read-only JavaBean properties (the ones without setters) should be included in
        the YAML document

        Returns
        - False when read-only JavaBean properties are not emitted
        """
        ...


    def setAllowReadOnlyProperties(self, allowReadOnlyProperties: bool) -> None:
        """
        Set to True to include read-only JavaBean properties (the ones without setters) in the YAML
        document. By default these properties are not included to be able to parse later the same
        JavaBean.

        Arguments
        - allowReadOnlyProperties: - True to dump read-only JavaBean properties
        """
        ...


    def getTimeZone(self) -> "TimeZone":
        """
        getter

        Returns
        - timezone to be used to emit Date
        """
        ...


    def setTimeZone(self, timeZone: "TimeZone") -> None:
        """
        Set the timezone to be used for Date. If set to `null` UTC is used.

        Arguments
        - timeZone: for created Dates or null to use UTC
        """
        ...


    def getAnchorGenerator(self) -> "AnchorGenerator":
        """
        getter

        Returns
        - generator to create anchor names
        """
        ...


    def setAnchorGenerator(self, anchorGenerator: "AnchorGenerator") -> None:
        """
        Provide a custom generator

        Arguments
        - anchorGenerator: - the way to create custom anchors
        """
        ...


    def getMaxSimpleKeyLength(self) -> int:
        ...


    def setMaxSimpleKeyLength(self, maxSimpleKeyLength: int) -> None:
        """
        Define max key length to use simple key (without '?') More info
        https://yaml.org/spec/1.1/#id934537

        Arguments
        - maxSimpleKeyLength: - the limit after which the key gets explicit key indicator '?'
        """
        ...


    def setProcessComments(self, processComments: bool) -> None:
        """
        Set the comment processing. By default, comments are ignored.

        Arguments
        - processComments: `True` to process; `False` to ignore
        """
        ...


    def isProcessComments(self) -> bool:
        """
        getter

        Returns
        - True when comments are not ignored and can be used after composing a Node
        """
        ...


    def getNonPrintableStyle(self) -> "NonPrintableStyle":
        ...


    def setNonPrintableStyle(self, style: "NonPrintableStyle") -> None:
        """
        When String contains non-printable characters SnakeYAML convert it to binary data with the
        !!binary tag. Set this to ESCAPE to keep the !!str tag and escape the non-printable chars with
        \\x or \\u

        Arguments
        - style: ESCAPE to force SnakeYAML to keep !!str tag for non-printable data
        """
        ...


    class ScalarStyle(Enum):
        """
        YAML provides a rich set of scalar styles. Block scalar styles include the literal style and
        the folded style; flow scalar styles include the plain style and two quoted styles, the
        single-quoted style and the double-quoted style. These styles offer a range of trade-offs
        between expressive power and readability.

        See
        - <a href="http://yaml.org/spec/1.1/.id858081">2.3. Scalars</a>
        """

        DOUBLE_QUOTED = ('"')
        """
        Double quoted scalar
        """
        SINGLE_QUOTED = ('\'')
        """
        Single quoted scalar
        """
        LITERAL = ('|')
        """
        Literal scalar
        """
        FOLDED = ('>')
        """
        Folded scalar
        """
        PLAIN = (None)
        """
        Plain scalar
        """


        def getChar(self) -> "Character":
            """
            getter

            Returns
            - the char behind the style
            """
            ...


        def toString(self) -> str:
            """
            Readable style

            Returns
            - for humans
            """
            ...


        @staticmethod
        def createStyle(style: "Character") -> "ScalarStyle":
            """
            Create

            Arguments
            - style: - source char

            Returns
            - parsed style
            """
            ...


    class FlowStyle(Enum):
        """
        Block styles use indentation to denote nesting and scope within the document. In contrast, flow
        styles rely on explicit indicators to denote nesting and scope.

        See
        - <a href="http://www.yaml.org/spec/current.html.id2509255">3.2.3.1. Node Styles
             (http://yaml.org/spec/1.1)</a>
        """

        FLOW = (True)
        """
        Flow style
        """
        BLOCK = (False)
        """
        Block style
        """
        AUTO = (None)
        """
        Auto (first block, than flow)
        """


        def toString(self) -> str:
            ...


    class LineBreak(Enum):
        """
        Platform dependent line break.
        """

        WIN = ("\r\n")
        """
        Windows
        """
        MAC = ("\r")
        """
        Old Mac (should not be used !)
        """
        UNIX = ("\n")
        """
        Linux and Mac
        """


        def getString(self) -> str:
            """
            getter

            Returns
            - the break
            """
            ...


        def toString(self) -> str:
            """
            for humans

            Returns
            - representation
            """
            ...


        @staticmethod
        def getPlatformLineBreak() -> "LineBreak":
            """
            Get the line break used by the current Operating System

            Returns
            - detected line break
            """
            ...


    class Version(Enum):
        """
        Specification version. Currently supported 1.0 and 1.1
        """

        V1_0 = (Integer[] { 1, 0 })
        """
        1.0
        """
        V1_1 = (Integer[] { 1, 1 })
        """
        1.1
        """


        def major(self) -> int:
            """
            getter

            Returns
            - major part (always 1)
            """
            ...


        def minor(self) -> int:
            """
            Minor part (0 or 1)

            Returns
            - 0 or 1
            """
            ...


        def getRepresentation(self) -> str:
            """
            getter

            Returns
            - representation for serialisation
            """
            ...


        def toString(self) -> str:
            """
            Readable string

            Returns
            - for humans
            """
            ...


    class NonPrintableStyle(Enum):
        """
        the way to serialize non-printable
        """

        BINARY = 0
        """
        Transform String to binary if it contains non-printable characters
        """
        ESCAPE = 1
        """
        Escape non-printable characters
        """



"""
Python module generated from Java source file org.bukkit.conversations.Conversation

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.conversations import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class Conversation:
    """
    The Conversation class is responsible for tracking the current state of a
    conversation, displaying prompts to the user, and dispatching the user's
    response to the appropriate place. Conversation objects are not typically
    instantiated directly. Instead a ConversationFactory is used to
    construct identical conversations on demand.
    
    Conversation flow consists of a directed graph of Prompt objects.
    Each time a prompt gets input from the user, it must return the next prompt
    in the graph. Since each Prompt chooses the next Prompt, complex
    conversation trees can be implemented where the nature of the player's
    response directs the flow of the conversation.
    
    Each conversation has a ConversationPrefix that prepends all output
    from the conversation to the player. The ConversationPrefix can be used to
    display the plugin name or conversation status as the conversation evolves.
    
    Each conversation has a timeout measured in the number of inactive seconds
    to wait before abandoning the conversation. If the inactivity timeout is
    reached, the conversation is abandoned and the user's incoming and outgoing
    chat is returned to normal.
    
    You should not construct a conversation manually. Instead, use the ConversationFactory for access to all available options.
    """

    def __init__(self, plugin: "Plugin", forWhom: "Conversable", firstPrompt: "Prompt"):
        """
        Initializes a new Conversation.

        Arguments
        - plugin: The plugin that owns this conversation.
        - forWhom: The entity for whom this conversation is mediating.
        - firstPrompt: The first prompt in the conversation graph.
        """
        ...


    def __init__(self, plugin: "Plugin", forWhom: "Conversable", firstPrompt: "Prompt", initialSessionData: dict["Object", "Object"]):
        """
        Initializes a new Conversation.

        Arguments
        - plugin: The plugin that owns this conversation.
        - forWhom: The entity for whom this conversation is mediating.
        - firstPrompt: The first prompt in the conversation graph.
        - initialSessionData: Any initial values to put in the conversation
            context sessionData map.
        """
        ...


    def getForWhom(self) -> "Conversable":
        """
        Gets the entity for whom this conversation is mediating.

        Returns
        - The entity.
        """
        ...


    def isModal(self) -> bool:
        """
        Gets the modality of this conversation. If a conversation is modal, all
        messages directed to the player are suppressed for the duration of the
        conversation.

        Returns
        - The conversation modality.
        """
        ...


    def isLocalEchoEnabled(self) -> bool:
        """
        Gets the status of local echo for this conversation. If local echo is
        enabled, any text submitted to a conversation gets echoed back into the
        submitter's chat window.

        Returns
        - The status of local echo.
        """
        ...


    def setLocalEchoEnabled(self, localEchoEnabled: bool) -> None:
        """
        Sets the status of local echo for this conversation. If local echo is
        enabled, any text submitted to a conversation gets echoed back into the
        submitter's chat window.

        Arguments
        - localEchoEnabled: The status of local echo.
        """
        ...


    def getPrefix(self) -> "ConversationPrefix":
        """
        Gets the ConversationPrefix that prepends all output from this
        conversation.

        Returns
        - The ConversationPrefix in use.
        """
        ...


    def getCancellers(self) -> list["ConversationCanceller"]:
        """
        Gets the list of ConversationCancellers

        Returns
        - The list.
        """
        ...


    def getContext(self) -> "ConversationContext":
        """
        Returns the Conversation's ConversationContext.

        Returns
        - The ConversationContext.
        """
        ...


    def begin(self) -> None:
        """
        Displays the first prompt of this conversation and begins redirecting
        the user's chat responses.
        """
        ...


    def getState(self) -> "ConversationState":
        """
        Returns Returns the current state of the conversation.

        Returns
        - The current state of the conversation.
        """
        ...


    def acceptInput(self, input: str) -> None:
        """
        Passes player input into the current prompt. The next prompt (as
        determined by the current prompt) is then displayed to the user.

        Arguments
        - input: The user's chat text.
        """
        ...


    def addConversationAbandonedListener(self, listener: "ConversationAbandonedListener") -> None:
        """
        Adds a ConversationAbandonedListener.

        Arguments
        - listener: The listener to add.
        """
        ...


    def removeConversationAbandonedListener(self, listener: "ConversationAbandonedListener") -> None:
        """
        Removes a ConversationAbandonedListener.

        Arguments
        - listener: The listener to remove.
        """
        ...


    def abandon(self) -> None:
        """
        Abandons and resets the current conversation. Restores the user's
        normal chat behavior.
        """
        ...


    def abandon(self, details: "ConversationAbandonedEvent") -> None:
        """
        Abandons and resets the current conversation. Restores the user's
        normal chat behavior.

        Arguments
        - details: Details about why the conversation was abandoned
        """
        ...


    def outputNextPrompt(self) -> None:
        """
        Displays the next user prompt and abandons the conversation if the next
        prompt is null.
        """
        ...


    class ConversationState(Enum):

        UNSTARTED = 0
        STARTED = 1
        ABANDONED = 2



"""
Python module generated from Java source file org.bukkit.block.data.type.Bed

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block.data import Bisected
from org.bukkit.block.data import Directional
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Bed(Directional):
    """
    Similar to Bisected, 'part' denotes which half of the bed this block
    corresponds to.
    
    'occupied' property is a quick flag to check if a player is currently
    sleeping in this bed block.
    """

    def getPart(self) -> "Part":
        """
        Gets the value of the 'part' property.

        Returns
        - the 'part' value
        """
        ...


    def setPart(self, part: "Part") -> None:
        """
        Sets the value of the 'part' property.

        Arguments
        - part: the new 'part' value
        """
        ...


    def isOccupied(self) -> bool:
        """
        Gets the value of the 'occupied' property.

        Returns
        - the 'occupied' value
        """
        ...


    class Part(Enum):
        """
        Horizontal half of a bed.
        """

        HEAD = 0
        """
        The head is the upper part of the bed containing the pillow.
        """
        FOOT = 1
        """
        The foot is the lower half of the bed.
        """



"""
Python module generated from Java source file org.bukkit.event.block.NotePlayEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Instrument
from org.bukkit import Note
from org.bukkit.block import Block
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from typing import Any, Callable, Iterable, Tuple


class NotePlayEvent(BlockEvent, Cancellable):
    """
    Called when a note block is being played through player interaction or a
    redstone current.
    """

    def __init__(self, block: "Block", instrument: "Instrument", note: "Note"):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getInstrument(self) -> "Instrument":
        """
        Gets the Instrument to be used.

        Returns
        - the Instrument
        """
        ...


    def getNote(self) -> "Note":
        """
        Gets the Note to be played.

        Returns
        - the Note
        """
        ...


    def setInstrument(self, instrument: "Instrument") -> None:
        """
        Overrides the Instrument to be used.

        Arguments
        - instrument: the Instrument. Has no effect if null.

        Deprecated
        - no effect on newer Minecraft versions
        """
        ...


    def setNote(self, note: "Note") -> None:
        """
        Overrides the Note to be played.

        Arguments
        - note: the Note. Has no effect if null.

        Deprecated
        - no effect on newer Minecraft versions
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.scanner.ScannerException

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.error import Mark
from org.yaml.snakeyaml.error import MarkedYAMLException
from org.yaml.snakeyaml.scanner import *
from typing import Any, Callable, Iterable, Tuple


class ScannerException(MarkedYAMLException):
    """
    Exception thrown by the Scanner implementations in case of malformed input.
    """

    def __init__(self, context: str, contextMark: "Mark", problem: str, problemMark: "Mark", note: str):
        """
        Constructs an instance.

        Arguments
        - context: Part of the input document in which vicinity the problem occurred.
        - contextMark: Position of the `context` within the document.
        - problem: Part of the input document that caused the problem.
        - problemMark: Position of the `problem` within the document.
        - note: Message for the user with further information about the problem.
        """
        ...


    def __init__(self, context: str, contextMark: "Mark", problem: str, problemMark: "Mark"):
        """
        Constructs an instance.

        Arguments
        - context: Part of the input document in which vicinity the problem occurred.
        - contextMark: Position of the `context` within the document.
        - problem: Part of the input document that caused the problem.
        - problemMark: Position of the `problem` within the document.
        """
        ...



"""
Python module generated from Java source file java.nio.file.Files

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import BufferedReader
from java.io import BufferedWriter
from java.io import Closeable
from java.io import File
from java.io import IOException
from java.io import InputStream
from java.io import InputStreamReader
from java.io import OutputStream
from java.io import OutputStreamWriter
from java.io import Reader
from java.io import UncheckedIOException
from java.io import Writer
from java.nio.charset import Charset
from java.nio.charset import CharsetDecoder
from java.nio.charset import CharsetEncoder
from java.nio.charset import StandardCharsets
from java.nio.file import *
from java.nio.file.attribute import BasicFileAttributeView
from java.nio.file.attribute import BasicFileAttributes
from java.nio.file.attribute import DosFileAttributes
from java.nio.file.attribute import FileAttribute
from java.nio.file.attribute import FileAttributeView
from java.nio.file.attribute import FileOwnerAttributeView
from java.nio.file.attribute import FileStoreAttributeView
from java.nio.file.attribute import FileTime
from java.nio.file.attribute import PosixFileAttributeView
from java.nio.file.attribute import PosixFileAttributes
from java.nio.file.attribute import PosixFilePermission
from java.nio.file.attribute import UserPrincipal
from java.nio.file.spi import FileSystemProvider
from java.nio.file.spi import FileTypeDetector
from java.security import AccessController
from java.security import PrivilegedAction
from java.util import Arrays
from java.util import Collections
from java.util import EnumSet
from java.util import Iterator
from java.util import Objects
from java.util import ServiceLoader
from java.util import Spliterator
from java.util.function import BiPredicate
from java.util.stream import Stream
from java.util.stream import StreamSupport
from jdk.internal.util import ArraysSupport
from sun.nio.ch import FileChannelImpl
from sun.nio.cs import UTF_8
from sun.nio.fs import ExtendedFileSystemProvider
from typing import Any, Callable, Iterable, Tuple


class Files:

    @staticmethod
    def newInputStream(path: "Path", *options: Tuple["OpenOption", ...]) -> "InputStream":
        ...


    @staticmethod
    def newOutputStream(path: "Path", *options: Tuple["OpenOption", ...]) -> "OutputStream":
        ...


    @staticmethod
    def newByteChannel(path: "Path", options: set["OpenOption"], *attrs: Tuple["FileAttribute"[Any], ...]) -> "SeekableByteChannel":
        ...


    @staticmethod
    def newByteChannel(path: "Path", *options: Tuple["OpenOption", ...]) -> "SeekableByteChannel":
        ...


    @staticmethod
    def newDirectoryStream(dir: "Path") -> "DirectoryStream"["Path"]:
        ...


    @staticmethod
    def newDirectoryStream(dir: "Path", glob: str) -> "DirectoryStream"["Path"]:
        ...


    @staticmethod
    def newDirectoryStream(dir: "Path", filter: "DirectoryStream.Filter"["Path"]) -> "DirectoryStream"["Path"]:
        ...


    @staticmethod
    def createFile(path: "Path", *attrs: Tuple["FileAttribute"[Any], ...]) -> "Path":
        ...


    @staticmethod
    def createDirectory(dir: "Path", *attrs: Tuple["FileAttribute"[Any], ...]) -> "Path":
        ...


    @staticmethod
    def createDirectories(dir: "Path", *attrs: Tuple["FileAttribute"[Any], ...]) -> "Path":
        ...


    @staticmethod
    def createTempFile(dir: "Path", prefix: str, suffix: str, *attrs: Tuple["FileAttribute"[Any], ...]) -> "Path":
        ...


    @staticmethod
    def createTempFile(prefix: str, suffix: str, *attrs: Tuple["FileAttribute"[Any], ...]) -> "Path":
        ...


    @staticmethod
    def createTempDirectory(dir: "Path", prefix: str, *attrs: Tuple["FileAttribute"[Any], ...]) -> "Path":
        ...


    @staticmethod
    def createTempDirectory(prefix: str, *attrs: Tuple["FileAttribute"[Any], ...]) -> "Path":
        ...


    @staticmethod
    def createSymbolicLink(link: "Path", target: "Path", *attrs: Tuple["FileAttribute"[Any], ...]) -> "Path":
        ...


    @staticmethod
    def createLink(link: "Path", existing: "Path") -> "Path":
        ...


    @staticmethod
    def delete(path: "Path") -> None:
        ...


    @staticmethod
    def deleteIfExists(path: "Path") -> bool:
        ...


    @staticmethod
    def copy(source: "Path", target: "Path", *options: Tuple["CopyOption", ...]) -> "Path":
        ...


    @staticmethod
    def move(source: "Path", target: "Path", *options: Tuple["CopyOption", ...]) -> "Path":
        ...


    @staticmethod
    def readSymbolicLink(link: "Path") -> "Path":
        ...


    @staticmethod
    def getFileStore(path: "Path") -> "FileStore":
        ...


    @staticmethod
    def isSameFile(path: "Path", path2: "Path") -> bool:
        ...


    @staticmethod
    def mismatch(path: "Path", path2: "Path") -> int:
        ...


    @staticmethod
    def isHidden(path: "Path") -> bool:
        ...


    @staticmethod
    def probeContentType(path: "Path") -> str:
        ...


    @staticmethod
    def getFileAttributeView(path: "Path", type: type["V"], *options: Tuple["LinkOption", ...]) -> "V":
        ...


    @staticmethod
    def readAttributes(path: "Path", type: type["A"], *options: Tuple["LinkOption", ...]) -> "A":
        ...


    @staticmethod
    def setAttribute(path: "Path", attribute: str, value: "Object", *options: Tuple["LinkOption", ...]) -> "Path":
        ...


    @staticmethod
    def getAttribute(path: "Path", attribute: str, *options: Tuple["LinkOption", ...]) -> "Object":
        ...


    @staticmethod
    def readAttributes(path: "Path", attributes: str, *options: Tuple["LinkOption", ...]) -> dict[str, "Object"]:
        ...


    @staticmethod
    def getPosixFilePermissions(path: "Path", *options: Tuple["LinkOption", ...]) -> set["PosixFilePermission"]:
        ...


    @staticmethod
    def setPosixFilePermissions(path: "Path", perms: set["PosixFilePermission"]) -> "Path":
        ...


    @staticmethod
    def getOwner(path: "Path", *options: Tuple["LinkOption", ...]) -> "UserPrincipal":
        ...


    @staticmethod
    def setOwner(path: "Path", owner: "UserPrincipal") -> "Path":
        ...


    @staticmethod
    def isSymbolicLink(path: "Path") -> bool:
        ...


    @staticmethod
    def isDirectory(path: "Path", *options: Tuple["LinkOption", ...]) -> bool:
        ...


    @staticmethod
    def isRegularFile(path: "Path", *options: Tuple["LinkOption", ...]) -> bool:
        ...


    @staticmethod
    def getLastModifiedTime(path: "Path", *options: Tuple["LinkOption", ...]) -> "FileTime":
        ...


    @staticmethod
    def setLastModifiedTime(path: "Path", time: "FileTime") -> "Path":
        ...


    @staticmethod
    def size(path: "Path") -> int:
        ...


    @staticmethod
    def exists(path: "Path", *options: Tuple["LinkOption", ...]) -> bool:
        ...


    @staticmethod
    def notExists(path: "Path", *options: Tuple["LinkOption", ...]) -> bool:
        ...


    @staticmethod
    def isReadable(path: "Path") -> bool:
        ...


    @staticmethod
    def isWritable(path: "Path") -> bool:
        ...


    @staticmethod
    def isExecutable(path: "Path") -> bool:
        ...


    @staticmethod
    def walkFileTree(start: "Path", options: set["FileVisitOption"], maxDepth: int, visitor: "FileVisitor"["Path"]) -> "Path":
        ...


    @staticmethod
    def walkFileTree(start: "Path", visitor: "FileVisitor"["Path"]) -> "Path":
        ...


    @staticmethod
    def newBufferedReader(path: "Path", cs: "Charset") -> "BufferedReader":
        ...


    @staticmethod
    def newBufferedReader(path: "Path") -> "BufferedReader":
        ...


    @staticmethod
    def newBufferedWriter(path: "Path", cs: "Charset", *options: Tuple["OpenOption", ...]) -> "BufferedWriter":
        ...


    @staticmethod
    def newBufferedWriter(path: "Path", *options: Tuple["OpenOption", ...]) -> "BufferedWriter":
        ...


    @staticmethod
    def copy(in: "InputStream", target: "Path", *options: Tuple["CopyOption", ...]) -> int:
        ...


    @staticmethod
    def copy(source: "Path", out: "OutputStream") -> int:
        ...


    @staticmethod
    def readAllBytes(path: "Path") -> list[int]:
        ...


    @staticmethod
    def readString(path: "Path") -> str:
        ...


    @staticmethod
    def readString(path: "Path", cs: "Charset") -> str:
        ...


    @staticmethod
    def readAllLines(path: "Path", cs: "Charset") -> list[str]:
        ...


    @staticmethod
    def readAllLines(path: "Path") -> list[str]:
        ...


    @staticmethod
    def write(path: "Path", bytes: list[int], *options: Tuple["OpenOption", ...]) -> "Path":
        ...


    @staticmethod
    def write(path: "Path", lines: Iterable["CharSequence"], cs: "Charset", *options: Tuple["OpenOption", ...]) -> "Path":
        ...


    @staticmethod
    def write(path: "Path", lines: Iterable["CharSequence"], *options: Tuple["OpenOption", ...]) -> "Path":
        ...


    @staticmethod
    def writeString(path: "Path", csq: "CharSequence", *options: Tuple["OpenOption", ...]) -> "Path":
        ...


    @staticmethod
    def writeString(path: "Path", csq: "CharSequence", cs: "Charset", *options: Tuple["OpenOption", ...]) -> "Path":
        ...


    @staticmethod
    def list(dir: "Path") -> "Stream"["Path"]:
        ...


    @staticmethod
    def walk(start: "Path", maxDepth: int, *options: Tuple["FileVisitOption", ...]) -> "Stream"["Path"]:
        ...


    @staticmethod
    def walk(start: "Path", *options: Tuple["FileVisitOption", ...]) -> "Stream"["Path"]:
        ...


    @staticmethod
    def find(start: "Path", maxDepth: int, matcher: "BiPredicate"["Path", "BasicFileAttributes"], *options: Tuple["FileVisitOption", ...]) -> "Stream"["Path"]:
        ...


    @staticmethod
    def lines(path: "Path", cs: "Charset") -> "Stream"[str]:
        ...


    @staticmethod
    def lines(path: "Path") -> "Stream"[str]:
        ...



"""
Python module generated from Java source file org.bukkit.scoreboard.DisplaySlot

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.scoreboard import *
from typing import Any, Callable, Iterable, Tuple


class DisplaySlot(Enum):
    """
    Locations for displaying objectives to the player
    """

    BELOW_NAME = 0
    PLAYER_LIST = 1
    SIDEBAR = 2
    SIDEBAR_BLACK = 3
    SIDEBAR_DARK_BLUE = 4
    SIDEBAR_DARK_GREEN = 5
    SIDEBAR_DARK_AQUA = 6
    SIDEBAR_DARK_RED = 7
    SIDEBAR_DARK_PURPLE = 8
    SIDEBAR_GOLD = 9
    SIDEBAR_GRAY = 10
    SIDEBAR_DARK_GRAY = 11
    SIDEBAR_BLUE = 12
    SIDEBAR_GREEN = 13
    SIDEBAR_AQUA = 14
    SIDEBAR_RED = 15
    SIDEBAR_LIGHT_PURPLE = 16
    SIDEBAR_YELLOW = 17
    SIDEBAR_WHITE = 18



"""
Python module generated from Java source file com.google.common.cache.CacheBuilder

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import Ascii
from com.google.common.base import Equivalence
from com.google.common.base import MoreObjects
from com.google.common.base import Supplier
from com.google.common.base import Suppliers
from com.google.common.base import Ticker
from com.google.common.cache import *
from com.google.common.cache.AbstractCache import SimpleStatsCounter
from com.google.common.cache.AbstractCache import StatsCounter
from com.google.common.cache.LocalCache import Strength
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.j2objc.annotations import J2ObjCIncompatible
from java.lang.ref import SoftReference
from java.lang.ref import WeakReference
from java.time import Duration
from java.util import ConcurrentModificationException
from java.util import IdentityHashMap
from java.util.concurrent import TimeUnit
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class CacheBuilder:
    """
    A builder of LoadingCache and Cache instances.
    
    <h2>Prefer <a href="https://github.com/ben-manes/caffeine/wiki">Caffeine</a> over Guava's caching
    API</h2>
    
    The successor to Guava's caching API is <a
    href="https://github.com/ben-manes/caffeine/wiki">Caffeine</a>. Its API is designed to make it a
    nearly drop-in replacement. It requires Java 8+, and is not available for Android or GWT/J2CL,
    and may have <a href="https://github.com/ben-manes/caffeine/wiki/Guava">different (usually
    better) behavior</a> when multiple threads attempt concurrent mutations. Its equivalent to `CacheBuilder` is its <a
    href="https://www.javadoc.io/doc/com.github.ben-manes.caffeine/caffeine/latest/com.github.benmanes.caffeine/com/github/benmanes/caffeine/cache/Caffeine.html">`Caffeine`</a> class. Caffeine offers better performance, more features (including asynchronous
    loading), and fewer <a
    href="https://github.com/google/guava/issues?q=is%3Aopen+is%3Aissue+label%3Apackage%3Dcache+label%3Atype%3Ddefect">bugs</a>.
    
    Caffeine defines its own interfaces (<a
    href="https://www.javadoc.io/doc/com.github.ben-manes.caffeine/caffeine/latest/com.github.benmanes.caffeine/com/github/benmanes/caffeine/cache/Cache.html">`Cache`</a>, <a
    href="https://www.javadoc.io/doc/com.github.ben-manes.caffeine/caffeine/latest/com.github.benmanes.caffeine/com/github/benmanes/caffeine/cache/LoadingCache.html">`LoadingCache`</a>, <a
    href="https://www.javadoc.io/doc/com.github.ben-manes.caffeine/caffeine/latest/com.github.benmanes.caffeine/com/github/benmanes/caffeine/cache/CacheLoader.html">`CacheLoader`</a>, etc.), so you can use Caffeine without needing to use any Guava types.
    Caffeine's types are better than Guava's, especially for <a
    href="https://www.javadoc.io/doc/com.github.ben-manes.caffeine/caffeine/latest/com.github.benmanes.caffeine/com/github/benmanes/caffeine/cache/AsyncLoadingCache.html">their
    deep support for asynchronous operations</a>. But if you want to migrate to Caffeine with minimal
    code changes, you can use <a
    href="https://www.javadoc.io/doc/com.github.ben-manes.caffeine/guava/latest/com.github.benmanes.caffeine.guava/com/github/benmanes/caffeine/guava/CaffeinatedGuava.html">its
    `CaffeinatedGuava` adapter class</a>, which lets you build a Guava `Cache` or a Guava
    `LoadingCache` backed by a Guava `CacheLoader`.
    
    Caffeine's API for asynchronous operations uses `CompletableFuture`: <a
    href="https://www.javadoc.io/doc/com.github.ben-manes.caffeine/caffeine/latest/com.github.benmanes.caffeine/com/github/benmanes/caffeine/cache/AsyncLoadingCache.html#get(K)">`AsyncLoadingCache.get`</a> returns a `CompletableFuture`, and implementations of <a
    href="https://www.javadoc.io/doc/com.github.ben-manes.caffeine/caffeine/latest/com.github.benmanes.caffeine/com/github/benmanes/caffeine/cache/AsyncCacheLoader.html#asyncLoad(K,java.util.concurrent.Executor)">`AsyncCacheLoader.asyncLoad`</a> must return a `CompletableFuture`. Users of Guava's com.google.common.util.concurrent.ListenableFuture can adapt between the two `Future`
    types by using <a href="https://github.com/lukas-krecan/future-converter#java8-guava">`net.javacrumbs.futureconverter.java8guava.FutureConverter`</a>.
    
    <h2>More on `CacheBuilder`</h2>
    
    `CacheBuilder` builds caches with any combination of the following features:
    
    
      - automatic loading of entries into the cache
      - least-recently-used eviction when a maximum size is exceeded (note that the cache is
          divided into segments, each of which does LRU internally)
      - time-based expiration of entries, measured since last access or last write
      - keys automatically wrapped in `WeakReference`
      - values automatically wrapped in `WeakReference` or `SoftReference`
      - notification of evicted (or otherwise removed) entries
      - accumulation of cache access statistics
    
    
    These features are all optional; caches can be created using all or none of them. By default,
    cache instances created by `CacheBuilder` will not perform any type of eviction.
    
    Usage example:
    
    ````LoadingCache<Key, Graph> graphs = CacheBuilder.newBuilder()
        .maximumSize(10000)
        .expireAfterWrite(Duration.ofMinutes(10))
        .removalListener(MY_LISTENER)
        .build(
            new CacheLoader<Key, Graph>() {
              public Graph load(Key key) throws AnyException {
                return createExpensiveGraph(key);`
            });
    }```
    
    Or equivalently,
    
    ````// In real life this would come from a command-line flag or config file
    String spec = "maximumSize=10000,expireAfterWrite=10m";
    
    LoadingCache<Key, Graph> graphs = CacheBuilder.from(spec)
        .removalListener(MY_LISTENER)
        .build(
            new CacheLoader<Key, Graph>() {
              public Graph load(Key key) throws AnyException {
                return createExpensiveGraph(key);`
            });
    }```
    
    The returned cache implements all optional operations of the LoadingCache and Cache interfaces. The `asMap` view (and its collection views) have *weakly consistent
    iterators*. This means that they are safe for concurrent use, but if other threads modify the
    cache after the iterator is created, it is undefined which of these changes, if any, are
    reflected in that iterator. These iterators never throw ConcurrentModificationException.
    
    **Note:** by default, the returned cache uses equality comparisons (the Object.equals equals method) to determine equality for keys or values. However, if .weakKeys was specified, the cache uses identity (`==`) comparisons instead for keys.
    Likewise, if .weakValues or .softValues was specified, the cache uses identity
    comparisons for values.
    
    Entries are automatically evicted from the cache when any of .maximumSize(long)
    maximumSize, .maximumWeight(long) maximumWeight, .expireAfterWrite
    expireAfterWrite, .expireAfterAccess expireAfterAccess, .weakKeys weakKeys,
    .weakValues weakValues, or .softValues softValues are requested.
    
    If .maximumSize(long) maximumSize or .maximumWeight(long) maximumWeight is
    requested entries may be evicted on each cache modification.
    
    If .expireAfterWrite expireAfterWrite or .expireAfterAccess expireAfterAccess
    is requested entries may be evicted on each cache modification, on occasional cache accesses, or
    on calls to Cache.cleanUp. Expired entries may be counted by Cache.size, but will
    never be visible to read or write operations.
    
    If .weakKeys weakKeys, .weakValues weakValues, or .softValues
    softValues are requested, it is possible for a key or value present in the cache to be reclaimed
    by the garbage collector. Entries with reclaimed keys or values may be removed from the cache on
    each cache modification, on occasional cache accesses, or on calls to Cache.cleanUp; such
    entries may be counted in Cache.size, but will never be visible to read or write
    operations.
    
    Certain cache configurations will result in the accrual of periodic maintenance tasks which
    will be performed during write operations, or during occasional read operations in the absence of
    writes. The Cache.cleanUp method of the returned cache will also perform maintenance, but
    calling it should not be necessary with a high throughput cache. Only caches built with .removalListener removalListener, .expireAfterWrite expireAfterWrite, .expireAfterAccess expireAfterAccess, .weakKeys weakKeys, .weakValues
    weakValues, or .softValues softValues perform periodic maintenance.
    
    The caches produced by `CacheBuilder` are serializable, and the deserialized caches
    retain all the configuration properties of the original cache. Note that the serialized form does
    *not* include cache contents, but only configuration.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/CachesExplained">caching</a> for a higher-level
    explanation.
    
    Type `<K>`: the most general key type this builder will be able to create caches for. This is
        normally `Object` unless it is constrained by using a method like `.removalListener`. Cache keys may not be null.
    
    Type `<V>`: the most general value type this builder will be able to create caches for. This is
        normally `Object` unless it is constrained by using a method like `.removalListener`. Cache values may not be null.

    Author(s)
    - Kevin Bourrillion

    Since
    - 10.0
    """

    @staticmethod
    def newBuilder() -> "CacheBuilder"["Object", "Object"]:
        """
        Constructs a new `CacheBuilder` instance with default settings, including strong keys,
        strong values, and no automatic eviction of any kind.
        
        Note that while this return type is `CacheBuilder<Object, Object>`, type parameters on
        the .build methods allow you to create a cache of any key and value type desired.
        """
        ...


    @staticmethod
    def from(spec: "CacheBuilderSpec") -> "CacheBuilder"["Object", "Object"]:
        """
        Constructs a new `CacheBuilder` instance with the settings specified in `spec`.

        Since
        - 12.0
        """
        ...


    @staticmethod
    def from(spec: str) -> "CacheBuilder"["Object", "Object"]:
        """
        Constructs a new `CacheBuilder` instance with the settings specified in `spec`.
        This is especially useful for command-line configuration of a `CacheBuilder`.

        Arguments
        - spec: a String in the format specified by CacheBuilderSpec

        Since
        - 12.0
        """
        ...


    def initialCapacity(self, initialCapacity: int) -> "CacheBuilder"["K", "V"]:
        """
        Sets the minimum total size for the internal hash tables. For example, if the initial capacity
        is `60`, and the concurrency level is `8`, then eight segments are created, each
        having a hash table of size eight. Providing a large enough estimate at construction time
        avoids the need for expensive resizing operations later, but setting this value unnecessarily
        high wastes memory.

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalArgumentException: if `initialCapacity` is negative
        - IllegalStateException: if an initial capacity was already set
        """
        ...


    def concurrencyLevel(self, concurrencyLevel: int) -> "CacheBuilder"["K", "V"]:
        """
        Guides the allowed concurrency among update operations. Used as a hint for internal sizing. The
        table is internally partitioned to try to permit the indicated number of concurrent updates
        without contention. Because assignment of entries to these partitions is not necessarily
        uniform, the actual concurrency observed may vary. Ideally, you should choose a value to
        accommodate as many threads as will ever concurrently modify the table. Using a significantly
        higher value than you need can waste space and time, and a significantly lower value can lead
        to thread contention. But overestimates and underestimates within an order of magnitude do not
        usually have much noticeable impact. A value of one permits only one thread to modify the cache
        at a time, but since read operations and cache loading computations can proceed concurrently,
        this still yields higher concurrency than full synchronization.
        
        Defaults to 4. **Note:**The default may change in the future. If you care about this
        value, you should always choose it explicitly.
        
        The current implementation uses the concurrency level to create a fixed number of hashtable
        segments, each governed by its own write lock. The segment lock is taken once for each explicit
        write, and twice for each cache loading computation (once prior to loading the new value, and
        once after loading completes). Much internal cache management is performed at the segment
        granularity. For example, access queues and write queues are kept per segment when they are
        required by the selected eviction algorithm. As such, when writing unit tests it is not
        uncommon to specify `concurrencyLevel(1)` in order to achieve more deterministic eviction
        behavior.
        
        Note that future implementations may abandon segment locking in favor of more advanced
        concurrency controls.

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalArgumentException: if `concurrencyLevel` is nonpositive
        - IllegalStateException: if a concurrency level was already set
        """
        ...


    def maximumSize(self, maximumSize: int) -> "CacheBuilder"["K", "V"]:
        """
        Specifies the maximum number of entries the cache may contain.
        
        Note that the cache **may evict an entry before this limit is exceeded**. For example, in
        the current implementation, when `concurrencyLevel` is greater than `1`, each
        resulting segment inside the cache *independently* limits its own size to approximately
        `maximumSize / concurrencyLevel`.
        
        When eviction is necessary, the cache evicts entries that are less likely to be used again.
        For example, the cache may evict an entry because it hasn't been used recently or very often.
        
        If `maximumSize` is zero, elements will be evicted immediately after being loaded into
        cache. This can be useful in testing, or to disable caching temporarily.
        
        This feature cannot be used in conjunction with .maximumWeight.

        Arguments
        - maximumSize: the maximum size of the cache

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalArgumentException: if `maximumSize` is negative
        - IllegalStateException: if a maximum size or weight was already set
        """
        ...


    def maximumWeight(self, maximumWeight: int) -> "CacheBuilder"["K", "V"]:
        """
        Specifies the maximum weight of entries the cache may contain. Weight is determined using the
        Weigher specified with .weigher, and use of this method requires a
        corresponding call to .weigher prior to calling .build.
        
        Note that the cache **may evict an entry before this limit is exceeded**. For example, in
        the current implementation, when `concurrencyLevel` is greater than `1`, each
        resulting segment inside the cache *independently* limits its own weight to approximately
        `maximumWeight / concurrencyLevel`.
        
        When eviction is necessary, the cache evicts entries that are less likely to be used again.
        For example, the cache may evict an entry because it hasn't been used recently or very often.
        
        If `maximumWeight` is zero, elements will be evicted immediately after being loaded
        into cache. This can be useful in testing, or to disable caching temporarily.
        
        Note that weight is only used to determine whether the cache is over capacity; it has no
        effect on selecting which entry should be evicted next.
        
        This feature cannot be used in conjunction with .maximumSize.

        Arguments
        - maximumWeight: the maximum total weight of entries the cache may contain

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalArgumentException: if `maximumWeight` is negative
        - IllegalStateException: if a maximum weight or size was already set

        Since
        - 11.0
        """
        ...


    def weigher(self, weigher: "Weigher"["K1", "V1"]) -> "CacheBuilder"["K1", "V1"]:
        """
        Specifies the weigher to use in determining the weight of entries. Entry weight is taken into
        consideration by .maximumWeight(long) when determining which entries to evict, and use
        of this method requires a corresponding call to .maximumWeight(long) prior to calling
        .build. Weights are measured and recorded when entries are inserted into the cache, and
        are thus effectively static during the lifetime of a cache entry.
        
        When the weight of an entry is zero it will not be considered for size-based eviction
        (though it still may be evicted by other means).
        
        **Important note:** Instead of returning *this* as a `CacheBuilder`
        instance, this method returns `CacheBuilder<K1, V1>`. From this point on, either the
        original reference or the returned reference may be used to complete configuration and build
        the cache, but only the "generic" one is type-safe. That is, it will properly prevent you from
        building caches whose key or value types are incompatible with the types accepted by the
        weigher already provided; the `CacheBuilder` type cannot do this. For best results,
        simply use the standard method-chaining idiom, as illustrated in the documentation at top,
        configuring a `CacheBuilder` and building your Cache all in a single statement.
        
        **Warning:** if you ignore the above advice, and use this `CacheBuilder` to build a
        cache whose key or value type is incompatible with the weigher, you will likely experience a
        ClassCastException at some *undefined* point in the future.

        Arguments
        - weigher: the weigher to use in calculating the weight of cache entries

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalStateException: if a weigher was already set or .maximumSize(long) was
            previously called

        Since
        - 11.0
        """
        ...


    def weakKeys(self) -> "CacheBuilder"["K", "V"]:
        """
        Specifies that each key (not value) stored in the cache should be wrapped in a WeakReference (by default, strong references are used).
        
        **Warning:** when this method is used, the resulting cache will use identity (`==`)
        comparison to determine equality of keys. Its Cache.asMap view will therefore
        technically violate the Map specification (in the same way that IdentityHashMap
        does).
        
        Entries with keys that have been garbage collected may be counted in Cache.size, but
        will never be visible to read or write operations; such entries are cleaned up as part of the
        routine maintenance described in the class javadoc.

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalStateException: if the key strength was already set
        """
        ...


    def weakValues(self) -> "CacheBuilder"["K", "V"]:
        """
        Specifies that each value (not key) stored in the cache should be wrapped in a WeakReference (by default, strong references are used).
        
        Weak values will be garbage collected once they are weakly reachable. This makes them a poor
        candidate for caching; consider .softValues instead.
        
        **Note:** when this method is used, the resulting cache will use identity (`==`)
        comparison to determine equality of values.
        
        Entries with values that have been garbage collected may be counted in Cache.size,
        but will never be visible to read or write operations; such entries are cleaned up as part of
        the routine maintenance described in the class javadoc.

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalStateException: if the value strength was already set
        """
        ...


    def softValues(self) -> "CacheBuilder"["K", "V"]:
        """
        Specifies that each value (not key) stored in the cache should be wrapped in a SoftReference (by default, strong references are used). Softly-referenced objects will be
        garbage-collected in a *globally* least-recently-used manner, in response to memory
        demand.
        
        **Warning:** in most circumstances it is better to set a per-cache .maximumSize(long) maximum size instead of using soft references. You should only use this
        method if you are well familiar with the practical consequences of soft references.
        
        **Note:** when this method is used, the resulting cache will use identity (`==`)
        comparison to determine equality of values.
        
        Entries with values that have been garbage collected may be counted in Cache.size,
        but will never be visible to read or write operations; such entries are cleaned up as part of
        the routine maintenance described in the class javadoc.

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalStateException: if the value strength was already set
        """
        ...


    def expireAfterWrite(self, duration: "Duration") -> "CacheBuilder"["K", "V"]:
        """
        Specifies that each entry should be automatically removed from the cache once a fixed duration
        has elapsed after the entry's creation, or the most recent replacement of its value.
        
        When `duration` is zero, this method hands off to .maximumSize(long)
        maximumSize`(0)`, ignoring any otherwise-specified maximum size or weight. This can be
        useful in testing, or to disable caching temporarily without a code change.
        
        Expired entries may be counted in Cache.size, but will never be visible to read or
        write operations. Expired entries are cleaned up as part of the routine maintenance described
        in the class javadoc.

        Arguments
        - duration: the length of time after an entry is created that it should be automatically
            removed

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalArgumentException: if `duration` is negative
        - IllegalStateException: if .expireAfterWrite was already set
        - ArithmeticException: for durations greater than +/- approximately 292 years

        Since
        - 25.0 (but only since 33.3.0 in the Android <a
            href="https://github.com/google/guava#guava-google-core-libraries-for-java">flavor</a>)
        """
        ...


    def expireAfterWrite(self, duration: int, unit: "TimeUnit") -> "CacheBuilder"["K", "V"]:
        """
        Specifies that each entry should be automatically removed from the cache once a fixed duration
        has elapsed after the entry's creation, or the most recent replacement of its value.
        
        When `duration` is zero, this method hands off to .maximumSize(long)
        maximumSize`(0)`, ignoring any otherwise-specified maximum size or weight. This can be
        useful in testing, or to disable caching temporarily without a code change.
        
        Expired entries may be counted in Cache.size, but will never be visible to read or
        write operations. Expired entries are cleaned up as part of the routine maintenance described
        in the class javadoc.
        
        If you can represent the duration as a Duration (which should be preferred when
        feasible), use .expireAfterWrite(Duration) instead.

        Arguments
        - duration: the length of time after an entry is created that it should be automatically
            removed
        - unit: the unit that `duration` is expressed in

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalArgumentException: if `duration` is negative
        - IllegalStateException: if .expireAfterWrite was already set
        """
        ...


    def expireAfterAccess(self, duration: "Duration") -> "CacheBuilder"["K", "V"]:
        """
        Specifies that each entry should be automatically removed from the cache once a fixed duration
        has elapsed after the entry's creation, the most recent replacement of its value, or its last
        access. Access time is reset by all cache read and write operations (including `Cache.asMap().get(Object)` and `Cache.asMap().put(K, V)`), but not by `containsKey(Object)`, nor by operations on the collection-views of Cache.asMap}. So,
        for example, iterating through `Cache.asMap().entrySet()` does not reset access time for
        the entries you retrieve.
        
        When `duration` is zero, this method hands off to .maximumSize(long)
        maximumSize`(0)`, ignoring any otherwise-specified maximum size or weight. This can be
        useful in testing, or to disable caching temporarily without a code change.
        
        Expired entries may be counted in Cache.size, but will never be visible to read or
        write operations. Expired entries are cleaned up as part of the routine maintenance described
        in the class javadoc.

        Arguments
        - duration: the length of time after an entry is last accessed that it should be
            automatically removed

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalArgumentException: if `duration` is negative
        - IllegalStateException: if .expireAfterAccess was already set
        - ArithmeticException: for durations greater than +/- approximately 292 years

        Since
        - 25.0 (but only since 33.3.0 in the Android <a
            href="https://github.com/google/guava#guava-google-core-libraries-for-java">flavor</a>)
        """
        ...


    def expireAfterAccess(self, duration: int, unit: "TimeUnit") -> "CacheBuilder"["K", "V"]:
        """
        Specifies that each entry should be automatically removed from the cache once a fixed duration
        has elapsed after the entry's creation, the most recent replacement of its value, or its last
        access. Access time is reset by all cache read and write operations (including `Cache.asMap().get(Object)` and `Cache.asMap().put(K, V)`), but not by `containsKey(Object)`, nor by operations on the collection-views of Cache.asMap. So, for
        example, iterating through `Cache.asMap().entrySet()` does not reset access time for the
        entries you retrieve.
        
        When `duration` is zero, this method hands off to .maximumSize(long)
        maximumSize`(0)`, ignoring any otherwise-specified maximum size or weight. This can be
        useful in testing, or to disable caching temporarily without a code change.
        
        Expired entries may be counted in Cache.size, but will never be visible to read or
        write operations. Expired entries are cleaned up as part of the routine maintenance described
        in the class javadoc.
        
        If you can represent the duration as a Duration (which should be preferred when
        feasible), use .expireAfterAccess(Duration) instead.

        Arguments
        - duration: the length of time after an entry is last accessed that it should be
            automatically removed
        - unit: the unit that `duration` is expressed in

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalArgumentException: if `duration` is negative
        - IllegalStateException: if .expireAfterAccess was already set
        """
        ...


    def refreshAfterWrite(self, duration: "Duration") -> "CacheBuilder"["K", "V"]:
        """
        Specifies that active entries are eligible for automatic refresh once a fixed duration has
        elapsed after the entry's creation, or the most recent replacement of its value. The semantics
        of refreshes are specified in LoadingCache.refresh, and are performed by calling CacheLoader.reload.
        
        As the default implementation of CacheLoader.reload is synchronous, it is
        recommended that users of this method override CacheLoader.reload with an asynchronous
        implementation; otherwise refreshes will be performed during unrelated cache read and write
        operations.
        
        Currently automatic refreshes are performed when the first stale request for an entry
        occurs. The request triggering refresh will make a synchronous call to CacheLoader.reload
        to obtain a future of the new value. If the returned future is already complete, it is returned
        immediately. Otherwise, the old value is returned.
        
        **Note:** *all exceptions thrown during refresh will be logged and then swallowed*.

        Arguments
        - duration: the length of time after an entry is created that it should be considered
            stale, and thus eligible for refresh

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalArgumentException: if `duration` is negative
        - IllegalStateException: if .refreshAfterWrite was already set
        - ArithmeticException: for durations greater than +/- approximately 292 years

        Since
        - 25.0 (but only since 33.3.0 in the Android <a
            href="https://github.com/google/guava#guava-google-core-libraries-for-java">flavor</a>)
        """
        ...


    def refreshAfterWrite(self, duration: int, unit: "TimeUnit") -> "CacheBuilder"["K", "V"]:
        """
        Specifies that active entries are eligible for automatic refresh once a fixed duration has
        elapsed after the entry's creation, or the most recent replacement of its value. The semantics
        of refreshes are specified in LoadingCache.refresh, and are performed by calling CacheLoader.reload.
        
        As the default implementation of CacheLoader.reload is synchronous, it is
        recommended that users of this method override CacheLoader.reload with an asynchronous
        implementation; otherwise refreshes will be performed during unrelated cache read and write
        operations.
        
        Currently automatic refreshes are performed when the first stale request for an entry
        occurs. The request triggering refresh will make a synchronous call to CacheLoader.reload
        and immediately return the new value if the returned future is complete, and the old value
        otherwise.
        
        **Note:** *all exceptions thrown during refresh will be logged and then swallowed*.
        
        If you can represent the duration as a Duration (which should be preferred when
        feasible), use .refreshAfterWrite(Duration) instead.

        Arguments
        - duration: the length of time after an entry is created that it should be considered
            stale, and thus eligible for refresh
        - unit: the unit that `duration` is expressed in

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalArgumentException: if `duration` is negative
        - IllegalStateException: if .refreshAfterWrite was already set

        Since
        - 11.0
        """
        ...


    def ticker(self, ticker: "Ticker") -> "CacheBuilder"["K", "V"]:
        """
        Specifies a nanosecond-precision time source for this cache. By default, System.nanoTime is used.
        
        The primary intent of this method is to facilitate testing of caches with a fake or mock
        time source.

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalStateException: if a ticker was already set
        """
        ...


    def removalListener(self, listener: "RemovalListener"["K1", "V1"]) -> "CacheBuilder"["K1", "V1"]:
        """
        Specifies a listener instance that caches should notify each time an entry is removed for any
        RemovalCause reason. Each cache created by this builder will invoke this listener
        as part of the routine maintenance described in the class documentation above.
        
        **Warning:** after invoking this method, do not continue to use *this* cache builder
        reference; instead use the reference this method *returns*. At runtime, these point to the
        same instance, but only the returned reference has the correct generic type information to
        ensure type safety. For best results, use the standard method-chaining idiom illustrated in the
        class documentation above, configuring a builder and building your cache in a single statement.
        Failure to heed this advice can result in a ClassCastException being thrown by a cache
        operation at some *undefined* point in the future.
        
        **Warning:** any exception thrown by `listener` will *not* be propagated to
        the `Cache` user, only logged via a Logger.

        Returns
        - this `CacheBuilder` instance (for chaining)

        Raises
        - IllegalStateException: if a removal listener was already set
        """
        ...


    def recordStats(self) -> "CacheBuilder"["K", "V"]:
        """
        Enable the accumulation of CacheStats during the operation of the cache. Without this
        Cache.stats will return zero for all statistics. Note that recording stats requires
        bookkeeping to be performed with each operation, and thus imposes a performance penalty on
        cache operation.

        Returns
        - this `CacheBuilder` instance (for chaining)

        Since
        - 12.0 (previously, stats collection was automatic)
        """
        ...


    def build(self, loader: "CacheLoader"["K1", "V1"]) -> "LoadingCache"["K1", "V1"]:
        """
        Builds a cache, which either returns an already-loaded value for a given key or atomically
        computes or retrieves it using the supplied `CacheLoader`. If another thread is currently
        loading the value for this key, simply waits for that thread to finish and returns its loaded
        value. Note that multiple threads can concurrently load values for distinct keys.
        
        This method does not alter the state of this `CacheBuilder` instance, so it can be
        invoked again to create multiple independent caches.

        Arguments
        - loader: the cache loader used to obtain new values

        Returns
        - a cache having the requested features
        """
        ...


    def build(self) -> "Cache"["K1", "V1"]:
        """
        Builds a cache which does not automatically load values when keys are requested.
        
        Consider .build(CacheLoader) instead, if it is feasible to implement a `CacheLoader`.
        
        This method does not alter the state of this `CacheBuilder` instance, so it can be
        invoked again to create multiple independent caches.

        Returns
        - a cache having the requested features

        Since
        - 11.0
        """
        ...


    def toString(self) -> str:
        """
        Returns a string representation for this CacheBuilder instance. The exact form of the returned
        string is not specified.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.inventory.InventoryInteractEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import HumanEntity
from org.bukkit.event import Cancellable
from org.bukkit.event.inventory import *
from org.bukkit.inventory import InventoryView
from typing import Any, Callable, Iterable, Tuple


class InventoryInteractEvent(InventoryEvent, Cancellable):
    """
    An abstract base class for events that describe an interaction between a
    HumanEntity and the contents of an Inventory.
    """

    def __init__(self, transaction: "InventoryView"):
        ...


    def getWhoClicked(self) -> "HumanEntity":
        """
        Gets the player who performed the click.

        Returns
        - The clicking player.
        """
        ...


    def setResult(self, newResult: "Result") -> None:
        """
        Sets the result of this event. This will change whether or not this
        event is considered cancelled.

        Arguments
        - newResult: the new org.bukkit.event.Event.Result for this event

        See
        - .isCancelled()
        """
        ...


    def getResult(self) -> "Result":
        """
        Gets the org.bukkit.event.Event.Result of this event. The Result describes the
        behavior that will be applied to the inventory in relation to this
        event.

        Returns
        - the Result of this event.
        """
        ...


    def isCancelled(self) -> bool:
        """
        Gets whether or not this event is cancelled. This is based off of the
        Result value returned by .getResult().  Result.ALLOW and
        Result.DEFAULT will result in a returned value of False, but
        Result.DENY will result in a returned value of True.
        

        Returns
        - whether the event is cancelled
        """
        ...


    def setCancelled(self, toCancel: bool) -> None:
        """
        Proxy method to .setResult(org.bukkit.event.Event.Result) for the Cancellable
        interface. .setResult(org.bukkit.event.Event.Result) is preferred, as it allows
        you to specify the Result beyond Result.DENY and Result.ALLOW.
        

        Arguments
        - toCancel: result becomes DENY if True, ALLOW if False
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.FluidLevelChangeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.block import Block
from org.bukkit.block.data import BlockData
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from typing import Any, Callable, Iterable, Tuple


class FluidLevelChangeEvent(BlockEvent, Cancellable):
    """
    Called when the fluid level of a block changes due to changes in adjacent
    blocks.
    """

    def __init__(self, theBlock: "Block", newData: "BlockData"):
        ...


    def getNewData(self) -> "BlockData":
        """
        Gets the new data of the changed block.

        Returns
        - new data
        """
        ...


    def setNewData(self, newData: "BlockData") -> None:
        """
        Sets the new data of the changed block. Must be of the same Material as
        the old one.

        Arguments
        - newData: the new data
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancelled: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.ListenableFutureTask

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util.concurrent import Callable
from java.util.concurrent import ExecutionException
from java.util.concurrent import Executor
from java.util.concurrent import FutureTask
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ListenableFutureTask(FutureTask, ListenableFuture):
    """
    A FutureTask that also implements the ListenableFuture interface. Unlike `FutureTask`, `ListenableFutureTask` does not provide an overrideable FutureTask.done() done() method. For similar functionality, call .addListener.
    
    Few users should use this class. It is intended primarily for those who are implementing an
    `ExecutorService`. Most users should call ListeningExecutorService.submit(Callable)
    ListeningExecutorService.submit on a service obtained from MoreExecutors.listeningDecorator.

    Author(s)
    - Sven Mawson

    Since
    - 1.0
    """

    @staticmethod
    def create(callable: "Callable"["V"]) -> "ListenableFutureTask"["V"]:
        """
        Creates a `ListenableFutureTask` that will upon running, execute the given `Callable`.

        Arguments
        - callable: the callable task

        Since
        - 10.0
        """
        ...


    @staticmethod
    def create(runnable: "Runnable", result: "V") -> "ListenableFutureTask"["V"]:
        """
        Creates a `ListenableFutureTask` that will upon running, execute the given `Runnable`, and arrange that `get` will return the given result on successful completion.

        Arguments
        - runnable: the runnable task
        - result: the result to return on successful completion. If you don't need a particular
            result, consider using constructions of the form: `ListenableFuture<?> f =
            ListenableFutureTask.create(runnable, null)`

        Since
        - 10.0
        """
        ...


    def addListener(self, listener: "Runnable", exec: "Executor") -> None:
        ...


    def get(self, timeout: int, unit: "TimeUnit") -> "V":
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.Runnables

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.util.concurrent import *
from typing import Any, Callable, Iterable, Tuple


class Runnables:
    """
    Static utility methods pertaining to the Runnable interface.

    Since
    - 16.0
    """

    @staticmethod
    def doNothing() -> "Runnable":
        """
        Returns a Runnable instance that does nothing when run.
        """
        ...



"""
Python module generated from Java source file org.joml.JvmciCode

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import *
from typing import Any, Callable, Iterable, Tuple



